import os
import re

from .utils import error, warn, info, get_app_id_by_domain, release_changes, \
    clear_changes, line
from .read_config import read_yaml_config, write_yaml_config

def replace_variables(client, content):
    if not isinstance(content, str):
        raise Exception("bad content")

    variables = []
    pattern = r'\$or_user_variable_[-a-zA-Z0-9_]+'
    variables = re.findall(pattern, content)

    variable_names = {}
    for var in variables:
        match = re.search(r'\$or_user_variable_(.*)', var)
        # should always match
        name = match.group(1)
        var_name =name
        variable_names[var] = var_name

    user_variables = client.get_all_user_vars()
    if not isinstance(user_variables, list):
        user_variables = list()

    user_variable_names = {}
    for var in user_variables:
        user_variable_names[var['name']] = var['id']

    for fake_name, real_name in variable_names.items():
        if real_name not in user_variable_names:
            raise Exception(f"user variable not found: {real_name}")

        var_id = user_variable_names[real_name]

        content = content.replace(fake_name, f'$or-{var_id}')

    return content

def restore_variable_names(client, content):
    if not isinstance(content, str):
        raise Exception("Bad content")

    # Find all global variable IDs in the content
    pattern = r'\$or-[0-9]+'
    content_variables = re.findall(pattern, content)

    # Get all global variables
    variables = client.get_all_user_vars()
    if not isinstance(variables, list):
        variables = []

    # Create a mapping of variable IDs to names
    variable_ids = {var['id']: var['name'] for var in variables}

    # Replace each variable ID with its corresponding name
    for var in content_variables:
        match = re.search(r'\$or-(\d+)', var)
        if match:
            var_id = int(match.group(1))
            if var_id in variable_ids:
                var_name = variable_ids[var_id]
                real_name = var_name
                if real_name is None:
                    error(f"Variable {var_name}(id: {var_id}) is not for the partition {partition_id}")

                content = content.replace(var, f'$or_user_variable_{real_name}')
            else:
                warn(f"Global variable ID not found: {var_id}")

    return content

def check_user_variables(variables, filename):
    if not isinstance(variables, list):
        error(f"Unsupported user variables file format, file: {filename}")

    valid_types = ['string', 'enum', 'num', 'int', 'bool']

    names = dict()
    for var in variables:
        if not isinstance(var, dict):
            error(f"Each user variable must be a dictionary, file: {filename}, line: {line(var)}")

        if 'name' not in var:
            error(f"Missing 'name' in user variable, file: {filename}, line: {line(var)}")

        if 'type' not in var:
            error(f"Missing 'type' in user variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['type'] not in valid_types:
            error(f"Invalid type '{var['type']}' for user variable {var['name']}, file: {filename}, line: {line(var)}")

        if 'default' not in var:
            error(f"Missing 'default' in user variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['type'] == 'enum':
            if 'values' not in var or not isinstance(var['values'], list):
                error(f"Missing or invalid 'values' for enum variable {var['name']}, file: {filename}, line: {line(var)}")
            if var['default'] not in var['values']:
                error(f"Default value not in enum values for variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['type'] == 'num':
            try:
                float(var['default'])
            except ValueError:
                error(f"Invalid default value for num variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['type'] == 'int':
            try:
                int(var['default'])
            except ValueError:
                error(f"Invalid default value for int variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['type'] == 'bool':
            if var['default'].lower() not in ['true', 'false']:
                error(f"Invalid default value for bool variable {var['name']}, file: {filename}, line: {line(var)}")

        if var['name'] in names:
            error(f"Duplicate name in user variable {var['name']}, file: {filename}, line: {line(var)}")

        names[var['name']] = True

    return True

def is_change_user_variable(new_var, old_var):
    if (new_var['name'] != old_var['name'] or
            new_var['type'] != old_var['type'] or
            new_var['default'] != old_var['default']):
        return True

    if new_var['type'] == 'enum' and new_var.get('values') != old_var.get('values'):
        return True

    return False

def process_user_variables(ctx):
    client = ctx['client']
    app_id = ctx['app_id']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)
    staging_release = ctx.get('staging_release', None)

    if not location:
        location = "user_variables"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if user variables are valid")
    for filename, variables in configs.items():
        check_user_variables(variables, filename)

    client.use_app(app_id)
    old_variables = client.get_all_user_vars()

    old_variables_dict = {var['name']: var for var in old_variables}

    info("Checking if user variables have changed")
    new_variables_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        variables = configs[filename]
        for var in variables:
            new_variables_dict[var['name']] = True

            if var['name'] in old_variables_dict:
                old_var = old_variables_dict[var['name']]
                if is_change_user_variable(var, old_var):
                    try:
                        info(f"Updating user variable \"{var['name']}\", app id: {app_id}")
                        client.put_user_var(
                            var_id=old_var['id'],
                            name=var['name'],
                            var_type=var['type'],
                            default=var['default'],
                            values=var.get('values')
                        )
                    except Exception as e:
                        clear_changes(client, app_id)
                        error(f"Failed to update user variable, file: {filename}, line: {line(var)}", e)
            else:
                try:
                    info(f"Adding user variable \"{var['name']}\" to app, app id: {app_id}")
                    client.new_user_var(
                        name=var['name'],
                        var_type=var['type'],
                        default=var['default'],
                        values=var.get('values')
                    )
                except Exception as e:
                    clear_changes(client, app_id)
                    error(f"Failed to add user variable to app, file: {filename}, line: {line(var)}", e)

    for var_name, var in old_variables_dict.items():
        if var_name not in new_variables_dict:
            try:
                info(f"Removing user variable \"{var_name}\" from app, app id: {app_id}")
                client.del_user_var(var['id'])
            except Exception as e:
                clear_changes(client, app_id)
                error(f"Failed to remove user variable from app, app id: {app_id}, variable id: {var['id']}", e)

    release_changes(client, app_id, staging_release=staging_release)

def cleanup_user_variables(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    staging_release = ctx.get('staging_release', None)

    app_id = get_app_id_by_domain(client, partition_id, domain)
    if not app_id:
        error(f"App not found, app id: {app_id}, domain: {domain}")

    client.use_app(app_id)
    user_variables = client.get_all_user_vars()

    for var in user_variables:
        try:
            info(f"Removing user variable \"{var['name']}\" from app, app id: {app_id}")
            client.del_user_var(var['id'])
        except Exception as e:
            clear_changes(client, app_id)
            error(f"Failed to remove user variable from app, app id: {app_id}, variable id: {var['id']}", e)

    release_changes(client, app_id, staging_release=staging_release)

def export_user_variables(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['export_to_path']

    if ctx.get('app_not_found', False) is True:
        return

    app_id = ctx.get('app_id', None)
    if app_id is None:
        app_id = get_app_id_by_domain(client, partition_id, domain)
        if not app_id:
            warn(f"App not found, partition_id: {partition_id}, domain: {domain}")
            return

        ctx['app_id'] = app_id

    client.use_app(app_id)
    user_variables = client.get_all_user_vars()

    if not user_variables:
        info(f"No user variables found for app_id: {app_id}")
        return

    formatted_variables = []
    for var in user_variables:
        formatted_var = {
            'name': var['name'],
            'type': var['type'],
            'default': var['default']
        }
        if var['type'] == 'enum':
            formatted_var['values'] = var['values']
        formatted_variables.append(formatted_var)

    export_path = os.path.join(configs_path, "user_variables")

    try:
        write_yaml_config(export_path, "user_variables.yaml", formatted_variables)
        info(f"User variables exported successfully to user_variables/user_variables.yaml")
    except Exception as e:
        error(f"Failed to export user variables to user_variables/user_variables.yaml", e)



# -*- coding: utf-8 -*-
import os
import string
import random
import time
import subprocess
import sdk_test

CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

class TestACMELogs(sdk_test.TestSdk):
    def test_002_acme_logs(self):
        init_script = os.path.join(CURPATH, 'tests', 'data', 'init-acme-logs.sh')

        app_id = self.app_id
        domains = [ self.apex ]

        cert_id = self.client.set_le_cert(domains=domains, acme_csr_type='rsa')
        self.assertIsInstance(cert_id, int)

        command = "/usr/bin/bash {} {}".format(init_script, cert_id)
        result = subprocess.run(command, shell=True, capture_output=True, text=True)
        print(result.stdout)
        if result.returncode != 0:
            print(result.stderr)
        self.assertEqual(result.returncode, 0)

        data, count = self.client.get_acme_logs(cert_id)
        self.assertEqual(len(data), 100)
        self.assertEqual(count, 100)



import os

from .utils import error, warn, info, line
from .read_config import read_yaml_config, write_yaml_config
from .k8s_upstreams import check_upstream_checker

def process_global_k8s_upstreams(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_k8s_upstreams"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    k8s_clusters = client.get_all_global_k8s()
    k8s_cluster_names = {cluster['name']: cluster['id'] for cluster in k8s_clusters}

    info("Checking if global k8s upstream is valid")
    for filename, ups in configs.items():
        check_global_k8s_upstreams(ups, filename, k8s_cluster_names)

    ups = convert_k8s_cluster_names(ups, k8s_cluster_names)

    old_upstreams = client.get_all_global_k8s_upstreams(detail=True)

    old_upstream_dict = {up['name']: up for up in old_upstreams}

    info("Checking if global k8s upstream have changed")
    new_upstream_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        ups = configs[filename]
        sorted_ups = sorted(ups.keys())
        for up_name in sorted_ups:
            new_up = ups[up_name]
            new_upstream_dict[up_name] = True

            checker = None
            if new_up.get('enable_checker', False):
                checker = new_up.get('checker', dict())

            if up_name in old_upstream_dict:
                old_up = old_upstream_dict[up_name]
                if is_change_global_k8s_upstream(new_up, old_up):
                    try:
                        info(f"Updating global k8s upstream \"{up_name}\"")
                        client.put_global_k8s_upstream(
                            up_id=old_up['id'],
                            name=up_name,
                            k8s_services=new_up['k8s_services'],
                            ssl=new_up.get('ssl', False),
                            disable_ssl_verify=new_up.get('disable_ssl_verify', False),
                            health_checker=checker,
                        )
                    except Exception as e:
                        error(f"Failed to update global k8s upstream, file: {filename}, line: {line(new_up)}", e)
            else:
                try:
                    info(f"Adding global k8s upstream \"{up_name}\"")
                    client.new_global_k8s_upstream(
                        name=up_name,
                        k8s_services=new_up['k8s_services'],
                        ssl=new_up.get('ssl', False),
                        disable_ssl_verify=new_up.get('disable_ssl_verify', False),
                        health_checker=checker
                    )
                except Exception as e:
                    error(f"Failed to add global k8s upstream, file: {filename}, line: {line(new_up)}", e)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # for up_name, up in old_upstream_dict.items():
    #     if up_name not in new_upstream_dict:
    #         try:
    #             info(f"Removing global k8s upstream \"{up_name}\"")
    #             client.del_global_k8s_upstream(up['id'])
    #         except Exception as e:
    #             error(f"Failed to remove global k8s upstream, upstream id: {up['id']}", e)

def cleanup_global_k8s_upstreams(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']

    # global_k8s_upstreams = client.get_all_global_k8s_upstreams(detail=False)

    # for up in global_k8s_upstreams:
    #     up_name = up['name']
    #     up_id = up['id']

    #     try:
    #         info(f"Removing global k8s upstream \"{up_name}\"(id: {up_id})")
    #         client.del_global_k8s_upstream(up_id)
    #     except Exception as e:
    #         error(f"Failed to remove global k8s upstream: {up_name}(id: {up_id})", e)

def export_global_k8s_upstreams(ctx):
    client = ctx['client']
    configs_path = ctx['export_to_path']

    global_k8s_upstreams = client.get_all_global_k8s_upstreams(detail=True)

    if not global_k8s_upstreams:
        info(f"No global k8s upstreams found")
        return

    k8s_clusters = client.get_all_global_k8s()
    k8s_cluster_ids = {cluster['id']: cluster['name'] for cluster in k8s_clusters}

    formatted_upstreams = {}
    for upstream in global_k8s_upstreams:
        upstream_name = upstream['name']
        k8s_services = upstream.get('k8s_services', [])
        new_k8s_services = []
        for service in k8s_services:
            k8s_name = k8s_cluster_ids.get(service['k8s'])
            if k8s_name is None:
                error(f"k8s cluster not found, k8s id: {service['k8s']}")

            new_service = {
                "k8s_name": k8s_name,
                "k8s_namespace": service['k8s_namespace'],
                "k8s_service": service['k8s_service'],
                "k8s_service_port": service['k8s_service_port'],
            }

            new_k8s_services.append(new_service)

        formatted_upstream = {
            'ssl': upstream.get('ssl', False),
            'disable_ssl_verify': upstream.get('disable_ssl_verify', False),
            'enable_checker': upstream.get('enable_checker', False),
            'k8s_services': new_k8s_services
        }

        if formatted_upstream['enable_checker']:
            formatted_upstream['checker'] = upstream.get('checker', {})

        formatted_upstreams[upstream_name] = formatted_upstream

    export_path = os.path.join(configs_path, "global_k8s_upstreams")

    try:
        write_yaml_config(export_path, "global_k8s_upstreams.yaml", formatted_upstreams)
        info(f"Global k8s upstreams exported successfully to global_k8s_upstreams/global_k8s_upstreams.yaml")
    except Exception as e:
        error(f"Failed to export global k8s upstreams to global_k8s_upstreams/global_k8s_upstreams.yaml", e)

def check_global_k8s_upstreams(upstreams, filename, k8s_cluster_names):
    if not isinstance(upstreams, dict):
        error(f"Unsupported global k8s upstream file format, file: {filename}")

    for up_name, up in upstreams.items():
        if not isinstance(up, dict):
            error(f"Global k8s upstream for name {up_name} must be a dict, file: {filename}, line: {line(up)}")

        ssl = up.get('ssl', False)
        if not isinstance(ssl, bool):
            error(f"ssl flag for global k8s upstream {up_name} must be a boolean, file: {filename}, line: {line(ssl)}")

        disable_ssl_verify = up.get('disable_ssl_verify', False)
        if not isinstance(disable_ssl_verify, bool):
            error(f"disable_ssl_verify flag for global k8s upstream {up_name} must be a boolean, file: {filename}, line: {line(disable_ssl_verify)}")

        check_upstream_checker(up, up_name, filename)

        k8s_services = up.get('k8s_services', None)
        if not isinstance(k8s_services, list) or not all(isinstance(item, dict) for item in k8s_services):
            error(f"k8s_services for global k8s upstream {up_name} must be a list of dictionaries, file: {filename}, line: {line(k8s_services)}")

        for s in k8s_services:
            required_keys = ['k8s_name', 'k8s_namespace', 'k8s_service', 'k8s_service_port']
            for key in required_keys:
                if key not in s:
                    error(f"Missing required key '{key}' in k8s service for global k8s upstream {up_name}, file: {filename}, line: {line(s)}")

            if not isinstance(s['k8s_service_port'], int) or not (1 <= s['k8s_service_port'] <= 65535):
                error(f"Invalid k8s_service_port in global k8s upstream {up_name}: {s['k8s_service_port']}, file: {filename}, line: {line(s)}")

            if s['k8s_name'] not in k8s_cluster_names:
                error(f"Invalid k8s_name in global k8s upstream {up_name}: {s['k8s_name']}, file: {filename}, line: {line(s)}")

def convert_k8s_cluster_names(upstreams, k8s_cluster_names):
    for up_name, up in upstreams.items():
        k8s_services = up.get('k8s_services', [])
        for s in k8s_services:
            s['k8s'] = k8s_cluster_names[s['k8s_name']]
    return upstreams

def is_change_global_k8s_upstream(new_up, old_up):
    if new_up.get('ssl', False) != old_up.get('ssl', False):
        return True

    if new_up.get('disable_ssl_verify', False) != old_up.get('disable_ssl_verify', False):
        return True

    if new_up.get('enable_checker', False) != old_up.get('enable_checker', False):
        return True

    if new_up.get('enable_checker', False) is True:
        new_checker = new_up.get('checker', {})
        old_checker = old_up.get('checker', {})
        for key in new_checker:
            if new_checker[key] != old_checker.get(key):
                return True

    new_services = new_up.get('k8s_services', [])
    old_services = old_up.get('k8s_services', [])
    if len(new_services) != len(old_services):
        return True

    for s1, s2 in zip(new_services, old_services):
        if (s1['k8s'] != s2['k8s']
                or s1['k8s_namespace'] != s2['k8s_namespace']
                or s1['k8s_service'] != s2['k8s_service']
                or s1['k8s_service_port'] != s2['k8s_service_port']):
            return True

    return False



# -*- coding: utf-8 -*-
import os
import sys
import random
import sdk_test


CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
HOST = os.getenv('EDGE_HOST')
USER_NAME = os.getenv('EDGE_USER')
PASSWORD = os.getenv('EDGE_PASSWORD')
sys.path.insert(0, CURPATH)


from edge2client import Edge2Client


class TestLogin(sdk_test.TestSdk):
    def test_login(self):
        ok = self.client.login()
        self.assertTrue(ok)
        if sys.version_info.major == 3:
            self.assertRegex(self.client.get_token(), r'^\S{32,}$')
        else:
            self.assertRegexpMatches(self.client.get_token(), r'^\S{32,}$')

    def test_login_with_wrong_username(self):
        client = Edge2Client(HOST, USER_NAME + 'wrong', PASSWORD)
        with self.assertRaises(Exception) as cm:
            client.login()

        self.assertEqual(str(cm.exception),
                         'failed to login: incorrect login credentials')



# -*- coding: utf-8 -*-
# import io
import os
import random
import sdk_test


CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


class TestRule(sdk_test.TestSdk):
    def test_rule_reorder(self):
        condition = [{'var': 'uri', 'op': 'eq', 'val': '/favicon.ico'}]
        conseq = [
            {'print': {
                'msg': 'hello'
            }}
        ]
        rule_id1 = self.client.new_rule(condition=condition, conseq=conseq, order=1)
        self.assertIs(type(rule_id1), int)
        self.assertGreater(rule_id1, 0)

        rule_id2 = self.client.new_rule(condition=condition, conseq=conseq, order=2)
        self.assertIs(type(rule_id2), int)
        self.assertGreater(rule_id2, 0)

        rule2 = self.client.get_rule(rule_id2)
        self.assertIs(type(rule2['order']), int)

        rule2_order = rule2['order']
        # insert rule after rule1, before rule 2
        rule_id3 = self.client.new_rule(condition=condition, conseq=conseq,
            order=rule2_order, reorder=True)
        self.assertIs(type(rule_id3), int)
        self.assertGreater(rule_id3, 0)

        # check rule2 order, it should have a new order
        rule2 = self.client.get_rule(rule_id2)
        self.assertEqual(rule2['order'], rule2_order + 1)

        # get rule3
        rule3 = self.client.get_rule(rule_id3)
        self.assertEqual(rule3['order'], rule2_order)

        # insert rule without reorder
        rule_id4 = self.client.new_rule(condition=condition, conseq=conseq,
            order=rule2_order)
        self.assertIs(type(rule_id3), int)
        self.assertGreater(rule_id3, 0)

        # get rule4
        rule4 = self.client.get_rule(rule_id3)
        self.assertEqual(rule4['order'], rule2_order)

        rules = self.client.get_all_rules()
        self.assertNotEqual(rule4['order'], rules[0]['order'])
        self.assertEqual(rule4['order'], rules[1]['order'])
        self.assertEqual(rule4['order'], rules[2]['order'])

        ok = self.client.del_rule(rule_id1)
        ok = self.client.del_rule(rule_id2)
        ok = self.client.del_rule(rule_id3)
        ok = self.client.del_rule(rule_id4)

        self.assertTrue(ok)



import os
import re

from .utils import error, warn, info, line
from .read_config import read_yaml_config, write_yaml_config, format_yaml_content

def process_global_page_templates(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_page_templates"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if page templates are valid")
    for filename, page_templates in configs.items():
        check_page_templates(page_templates, filename)

    old_page_templates = client.get_all_global_page_templates()

    old_page_templates_dict = {template['name']: template for template in old_page_templates}

    info("Checking if page templates have changed")
    new_page_templates_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        page_templates = configs[filename]
        for template in page_templates:
            new_page_templates_dict[template['name']] = True

            if template['name'] in old_page_templates_dict:
                old_template = old_page_templates_dict[template['name']]
                if is_change_page_template(template, old_template):
                    try:
                        info(f"Updating Global page template \"{template['name']}\"")
                        client.put_global_page_template(
                            id=old_template['id'],
                            name=template['name'],
                            content=template['content']
                        )
                    except Exception as e:
                        error(f"Failed to update page template, file: {filename}, line: {line(template)}", e)
            else:
                try:
                    info(f"Adding global page template \"{template['name']}\"")
                    client.new_global_page_template(
                        name=template['name'],
                        content=template['content']
                    )
                except Exception as e:
                    error(f"Failed to add page template, file: {filename}, line: {line(template)}", e)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # for template_name, template in old_page_templates_dict.items():
    #     if template_name not in new_page_templates_dict:
    #         try:
    #             info(f"Removing Global page template \"{template_name}\"")
    #             client.del_global_page_template(template['id'])
    #         except Exception as e:
    #             error(f"Failed to remove Global page template, template id: {template['id']}", e)

def cleanup_global_page_templates(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']
    # partition_id = ctx['partition_id']

    # page_templates = client.get_all_global_page_templates()

    # for template in page_templates:
    #     try:
    #         info(f"Removing Global page template \"{template['name']}\"")
    #         client.del_global_page_template(template['id'])
    #     except Exception as e:
    #         error(f"Failed to remove Global page template, template id: {template['id']}", e)

def export_global_page_templates(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['export_to_path']

    page_templates = client.get_all_global_page_templates()

    if not page_templates:
        info(f"No Global page templates found")
        return


    formatted_page_templates = []
    for template in page_templates:
        formatted_template = {
            'name': template['name'],
            'content': format_yaml_content(template['content'])
        }
        formatted_page_templates.append(formatted_template)

    export_path = os.path.join(configs_path, "global_page_templates")

    try:
        write_yaml_config(export_path, "global_page_templates.yaml", formatted_page_templates)
        info(f"Page templates exported successfully to global_page_templates/global_page_templates.yaml")
    except Exception as e:
        error(f"Failed to export page templates to global_page_templates/global_page_templates.yaml", e)

def check_page_templates(page_templates, filename):
    for template in page_templates:
        if 'name' not in template:
            error(f"Page template missing 'name' field in file: {filename}, line: {line(template)}")
        if 'content' not in template:
            error(f"Page template missing 'content' field in file: {filename}, line: {line(template)}")

def is_change_page_template(new_template, old_template):
    return new_template['content'] != old_template['content']

def name_to_id(client, content):
    if not isinstance(content, str):
        raise Exception("bad content")

    matched_items = []
    pattern = r'\$or_global_page_template_[a-zA-Z0-9_-]+'
    matched_items = re.findall(pattern, content)
    unique_matched_items = list(set(matched_items))

    names = {}
    for var in unique_matched_items:
        match = re.search(r'\$or_global_page_template_(.*)', var)
        # should always match
        name = match.group(1)
        names[var] = name

    templates = client.get_all_global_page_templates()
    if not isinstance(templates, list):
        templates = list()

    template_names = {}
    for t in templates:
        template_names[t['name']] = t['id']

    data = client.get_sync_to_all()
    ngx_sync_to_all = True
    if data and data.get('ngx', True) == False:
        ngx_sync_to_all = False

    for fake_name, name in names.items():
        if name in template_names:
            var_id = template_names[name]
        else:
            raise Exception(f"global page templates not found in Edge Admin: {name}")

        content = content.replace(fake_name, f'{var_id}')

    return content

def replace_global_page_template_in_el(client, actions, filename):
    if not isinstance(actions, list) and not isinstance(actions, dict):
        return

    if not isinstance(actions, list) and isinstance(actions, dict):
        new_actions = list()
        for k, v in actions.items():
            new_actions.append({k: v})
        actions = new_actions

    for action in actions:
        if not isinstance(action, dict) or 'user-code' not in action:
            continue

        el_value = ''
        user_code_data = action['user-code']
        if isinstance(user_code_data, dict) and 'el' in user_code_data:
            el_value = user_code_data['el']
        else:
            if user_code_data:
                error(f"bad action in page rule, file: {filename}, line: {line(user_code_data)}")
            else:
                error(f"bad action in page rule, file: {filename}, line: {line(user_code_data)}")

        try:
            user_code_data['el'] = name_to_id(client, el_value)
        except Exception as e:
            error(f"failed to replace global variable in Edgelang, file: {filename}, line: {line(user_code_data)}", e)

def id_to_name(client, content):
    if not isinstance(content, str):
        raise Exception("Bad content")

    pattern = r'page-template-id:\s*\d+'
    matched_items = re.findall(pattern, content)

    templates = client.get_all_global_page_templates()
    if not isinstance(templates, list):
        templates = []

    template_ids = {var['id']: var['name'] for var in templates}

    for var in matched_items:
        match = re.search(r'page-template-id:\s*(\d+)', var)
        if match:
            var_id = int(match.group(1))
            if var_id in template_ids:
                var_name = template_ids[var_id]
                content = content.replace(var, f'page-template-id: $or_global_page_template_{var_name}')
            else:
                warn(f"Global page template ID not found: {var_id}")

    return content

def restore_global_page_template_names_in_el(client, actions):
    if not isinstance(actions, list) and not isinstance(actions, dict):
        return

    if not isinstance(actions, list) and isinstance(actions, dict):
        new_actions = list()
        for k, v in actions.items():
            new_actions.append({k: v})
        actions = new_actions

    for action in actions:
        if not isinstance(action, dict) or 'user-code' not in action:
            continue

        user_code_data = action['user-code']
        if isinstance(user_code_data, dict) and 'el' in user_code_data:
            el_value = user_code_data['el']
            try:
                user_code_data['el'] = id_to_name(client, el_value)
            except Exception as e:
                error(f"Failed to restore global page template names in Edgelang", e)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
update app's upstream weight
"""

import argparse
import re
from math import gcd, ceil

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def group_weight_parser(string):
    arr = string.split(':')

    if len(arr) != 2:
        msg = 'should be <group>:<weight> format but "%r" found' % string
        raise argparse.ArgumentTypeError(msg)

    return arr


def upstream_name_parser(s):
    result = re.match('(.*)-([^-]+)$', s)

    if result is None:
        msg = '%r is not well formed upstream_name' % s
        print(msg)
        exit(1)

    return result


def conv_proxy(old_proxy, new_upstreams):
    return {
        'upstreams': new_upstreams,
    }


def find_upstream_by_id(up_id, upstreams):
    for up in upstreams:
        if up_id == up.get('cluster'):
            return True

    return False


def parse_group_weight_dict(group_weight):
    group_weight_dict = {}

    weight_gcd = None
    last_weight = None
    for gw in group_weight:
        group = gw[0]
        weight = int(gw[1])
        group_weight_dict[group] = weight
        if weight_gcd is None:
            weight_gcd = weight
        if last_weight is not None:
            weight_gcd = gcd(weight, weight_gcd)
        last_weight = weight

    max_weight = 0

    for group in group_weight_dict.keys():
        if weight_gcd > 0:
            new_weight = int(group_weight_dict[group] / weight_gcd)
            group_weight_dict[group] = new_weight

            if new_weight > max_weight:
                max_weight = new_weight

    if max_weight > 20:
        roughly_scale = ceil(max_weight / 20)

        for group in group_weight_dict.keys():
            new_weight = ceil(group_weight_dict[group] / roughly_scale)

            if new_weight < 0:
                new_weight = 0
            elif new_weight > 20:
                new_weight = 20

            group_weight_dict[group] = new_weight

    return group_weight_dict


def get_proxy_rules(client):
    rules = client.get_all_rules()

    proxy_rules = []

    for rule in rules:
        if rule.get('proxy'):
            old_rule = rule

            proxy_rule = {}
            for k in old_rule.keys():
                if k.startswith('_'):
                    continue
                proxy_rule[k] = old_rule[k]

            proxy_rules.append(proxy_rule)

    return proxy_rules


def modify_proxy_rule(client, args, proxy_rule,
                      group_weight_dict, all_upstreams):
    proxy_rule_id = proxy_rule.get('id')

    upstreams = []
    old_proxy = proxy_rule.get('proxy')
    if not old_proxy:
        print('there is no proxy rules.')
        exit(1)

    id_weight_dict = {}
    for up in old_proxy.get('upstream', []):
        up_id = up.get('cluster')
        upstream = client.get_upstream(up_id)
        id_weight_dict[up_id] = up.get('weight', 0)
        upstreams.append(upstream)

    new_upstreams = []
    serv_names = []
    unchanged_weight_cnt = 0

    for up in upstreams:
        up_name = up.get('name')
        up_group = up.get('group')

        if up_group is None or up_group == '':
            print('>>> WARN: upstream "{}" does not have a group name.'
                  .format(up_name), flush=True)
            up_id = up.get('id')
            weight = id_weight_dict.get(up_id, 0)

            print('id: {}, upstream_name: {}, weight: {}'
                  .format(up_id, up_name, weight),
                  flush=True)

            unchanged_weight_cnt += 1
            new_upstreams.append({'upstream': up_id, 'weight': weight})
            continue

        ret = upstream_name_parser(up_name)
        serv_name = ret.group(1)
        group_name = ret.group(2)

        if up_group != group_name:
            print('"{}" (group: {}) is not a well-formed upstream_name'
                  .format(up_name, up_group))
            exit(1)

        if serv_name not in serv_names:
            serv_names.append(serv_name)

    for grp in group_weight_dict.keys():
        weight = group_weight_dict.get(grp)
        if weight is None:
            continue

        for serv_name in serv_names:
            up_name = '{}-{}'.format(serv_name, grp)
            up_id = all_upstreams.get(up_name)
            old_weight = id_weight_dict.get(up_id, 0)
            if not up_id:
                print('>>> WARN: group: {}, upstream_name: {} does not exists.'
                      .format(grp, up_name), flush=True)
                continue

            if old_weight == weight:
                unchanged_weight_cnt += 1
            print('id: {}, upstream_name: {}, group: {}, weight: {} -> {}'
                  .format(up_id, serv_name, grp, old_weight, weight),
                  flush=True)

            if find_upstream_by_id(up_id, old_proxy.get('upstream', [])):
                new_upstreams.append({'upstream': up_id, 'weight': weight})
                continue

            # found which group
            for tmp_grp in group_weight_dict.keys():
                tmp_up_id = all_upstreams.get('{}-{}'
                                              .format(serv_name, tmp_grp))
                if find_upstream_by_id(tmp_up_id, old_proxy.get('upstream')):
                    new_upstreams.append({'upstream': up_id, 'weight': weight})
                    break

    # diff upstream
    for up in upstreams:
        up_id = up.get('id')
        found = False

        for nup in new_upstreams:
            if up_id == nup.get('upstream'):
                found = True
                break

        if not found:
            weight = id_weight_dict.get(up_id, 0)

            print('id: {}, upstream_name: {}, weight: {}'
                  .format(up_id, up.get('name'), weight),
                  flush=True)

            unchanged_weight_cnt += 1
            new_upstreams.append({'upstream': up_id, 'weight': weight})

    weight_zero_cnt = 0
    # check new_upstreams before send
    for nup in new_upstreams:
        if nup.get('weight') == 0:
            weight_zero_cnt += 1

    if weight_zero_cnt == len(new_upstreams):
        print('!!!ERROR: all upstream weight is zero, '
              'exiting...')
        exit(1)

    new_proxy = conv_proxy(old_proxy, new_upstreams)

    if len(new_upstreams) == unchanged_weight_cnt:
        print('nothing changed in this rule, skip...')
        return

    if args.dry_run:
        print('dry run here. exiting...')
        return

    ok = client.patch_rule(proxy_rule_id, proxy=new_proxy)
    if ok:
        print('modified upstream group weight successfully.', flush=True)
    else:
        print('something wrong happend.', flush=True)


def main(args):
    # init
    client = py_client.get_client()
    client.use_app(args.app_id)

    group_weight_dict = parse_group_weight_dict(args.group_weight)

    proxy_rules = get_proxy_rules(client)
    all_upstreams = client.get_all_upstreams()

    for proxy_rule in proxy_rules:
        print("{} rule_id: {} {}"
              .format('-' * 10, proxy_rule.get('id'), '-' * 10), flush=True)
        modify_proxy_rule(client, args, proxy_rule,
                          group_weight_dict, all_upstreams)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='update upstream group weight')

    parser.add_argument('--dry-run',
                        dest='dry_run',
                        action='store_true',
                        help='dry run.')
    parser.add_argument('--app-id',
                        dest='app_id',
                        type=int,
                        required=True,
                        help='specifiy the app_id to operate on.')
    parser.add_argument('--group-weight',
                        dest='group_weight',
                        type=group_weight_parser,
                        required=True,
                        nargs='+',
                        help='upstream group:weight')

    args = parser.parse_args()

    main(args)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
This file is used to export csv from api
"""

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def main():
    client = py_client.get_client()
    data = client.get_all_apps(detail=True)

    cached_partitions = {}

    print("domain,port,partition,label")
    for info in data.values():
        csv_table = {
            'domain': [],
            'port': [],
            'partition': [],
            'label': None
        }
        csv_table['label'] = info['label']
        for domain in info['domains']:
            csv_table['domain'].append(domain['domain'])
        if info['http_ports']:
            for port in info['http_ports']:
                csv_table['port'].append(str(port))
        if info['https_ports']:
            for port in info['https_ports']:
                csv_table['port'].append(str(port))
        for partition in info['partitions']:
            if partition in cached_partitions:
                pass
            else:
                partition_data = client.get_cluster_group(partition)
                cached_partitions[partition] = partition_data['name']

            csv_table['partition'].append(cached_partitions[partition])
            print(u"{},{},{},{}".format(
                "|".join(csv_table['domain']),
                "|".join(csv_table['port']),
                "|".join(csv_table['partition']),
                csv_table['label']
            ))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# import io
import sdk_test


class TestIPList(sdk_test.TestSdk):
    def test_global_ip_list(self):
        # create
        global_rule_id = self.client.new_global_ip_list(name='g_ip_list_1',
                                                        items=[{'ip': '127.0.0.2'}])
        self.assertIsInstance(global_rule_id, int)

        # get
        g_ip_list_1 = self.client.get_global_ip_list(global_rule_id)
        self.assertEqual(g_ip_list_1.get('name'), 'g_ip_list_1')

        # modify
        self.client.put_global_ip_list(rule_id=global_rule_id,
                                       items=[{'ip': '192.168.1.2'}])
        g_ip_list_1 = self.client.get_global_ip_list(global_rule_id)
        self.assertEqual(g_ip_list_1.get('items')[0].get('ip'), '192.168.1.2')

        # append
        self.client.append_to_global_ip_list(rule_id=global_rule_id, items=[{'ip': '192.168.1.3'}])
        g_ip_list_2 = self.client.get_global_ip_list(global_rule_id)
        self.assertEqual(g_ip_list_2.get('items')[0].get('ip'), '192.168.1.2')
        self.assertEqual(g_ip_list_2.get('items')[1].get('ip'), '192.168.1.3')

        # remove
        self.client.remove_from_global_ip_list(rule_id=global_rule_id,
                                        items=[{'ip': '192.168.1.2'}])
        g_ip_list_3 = self.client.get_global_ip_list(global_rule_id)
        self.assertEqual(g_ip_list_3.get('items')[0].get('ip'), '192.168.1.3')
        self.assertEqual(1, len(g_ip_list_3.get('items', [])))

        # delete
        ok = self.client.del_global_ip_list(rule_id=global_rule_id)
        self.assertTrue(ok)

    def test_global_ip_list2(self):
        # delete all global ip lists first
        ip_lists = self.client.get_all_global_ip_lists(detail=False)
        if ip_lists:
            for ip_list in ip_lists:
                self.client.del_global_ip_list(rule_id=ip_list['id'])

        # create the first ip list
        global_rule_id = self.client.new_global_ip_list(name='g_ip_list_1',
                                                        items=[{'ip': '127.0.0.2'}, {'ip': '127.0.0.3'}])
        self.assertIsInstance(global_rule_id, int)

        # create the second ip list
        global_rule_id2 = self.client.new_global_ip_list(name='g_ip_list_2',
                                                        items=[{'ip': '127.0.0.4'}])
        self.assertIsInstance(global_rule_id2, int)

        ip_lists = self.client.get_all_global_ip_lists()
        self.assertEqual(len(ip_lists), 2)
        self.assertEqual(ip_lists[0]['id'], global_rule_id)
        self.assertEqual(ip_lists[1]['id'], global_rule_id2)

        ip_list_1 = ip_lists[0]
        ip_list_2 = ip_lists[1]
        self.assertEqual(ip_list_1['items'][0]['ip'], '127.0.0.2')
        self.assertEqual(ip_list_1['items'][1]['ip'], '127.0.0.3')
        self.assertEqual(ip_list_2['items'][0]['ip'], '127.0.0.4')

        # delete
        ok = self.client.del_global_ip_list(rule_id=global_rule_id)
        self.assertTrue(ok)

        # delete
        ok = self.client.del_global_ip_list(rule_id=global_rule_id2)
        self.assertTrue(ok)



import os

from .utils import error, warn, info, generate_certificates, \
    get_app_id_by_domain, line, clear_changes, release_changes
from .read_config import read_yaml_file, write_yaml_config

def add_default_ssl_cert(client, app_id, domains, domain):
    rootCA_key, rootCA_crt, edge_node_key, edge_node_csr, edge_node_crt = generate_certificates(domains, domain)

    client.use_app(app_id)
    client.set_cert_key(cert=edge_node_crt, key=edge_node_key)

def add_http_app(client, partition_id, domains, cn_domain, access_log_name, staging_release):
    domains_str = ",".join(domains)
    info(f"adding http application: {domains_str}, partition id: {partition_id}")
    access_log = dict()
    if access_log_name:
        access_log = {
            'filename': "access.log",
            'name': access_log_name,
        }
    http_ports = [80]
    https_ports = [443]
    partition_ids = [partition_id]

    try:
        app_id = client.new_app(domains=domains, cluster_groups=partition_ids,
                                access_log=access_log, http_ports=http_ports,
                                https_ports=https_ports)
    except Exception as e:
        # just warning, do not interrupt the execution
        warn(f"failed to create app: {e}")

    try:
        info(f"adding ssl cert to app, app id: {app_id}, domain: {domains_str}")
        add_default_ssl_cert(client, app_id, domains, cn_domain)
    except Exception as e:
        # just warning, do not interrupt the execution
        warn(f"failed to ssl cert to app, app id: {app_id}, error: {e}")

    release_changes(client, app_id, staging_release=staging_release)
    return app_id

def process_http_app(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['configs_path']
    staging_release = ctx.get('staging_release', None)

    if not domain:
        error("missing domain.")

    log_format = None
    app_config = None

    file = f"{configs_path}/app.yaml"
    if os.path.exists(file):
        app_config = read_yaml_file(file)

    if app_config and 'access_log_format' in app_config:
        # use the first domain
        log_format = app_config['access_log_format']

    domains = [domain]
    if app_config and 'domains' in app_config:
        app_domains = app_config['domains']
        for dn in app_domains:
            if dn != domain:
                domains.append(dn)

    # get app in partition
    app_id = get_app_id_by_domain(client, partition_id, domain)
    if app_id:
        client.use_app(app_id)

        need_release = False
        if log_format is not None:
            # if this configuration is not specified in the configuration file,
            # then we will not make changes
            data = client.get_app_config(app_id)
            cur_log_format = data.get('access_log', dict())
            cur_log_format_name = cur_log_format.get('name', None)
            if cur_log_format_name is None or cur_log_format_name != log_format:
                try:
                    info(f"updating app config, app id: {app_id}, file: app.yaml, line: {line(app_config)}")
                    new_log_format = {'name': log_format, 'filename': 'access.log'}
                    client.put_app_config(app_id, access_log=new_log_format)
                    need_release = True
                except Exception as e:
                    clear_changes(client, app_id)
                    error(f"failed to update app config, app id: {app_id}, file: app.yaml, line: {line(app_config)}, error: {e}")

        if app_config and 'domains' in app_config:
            # if this configuration is not specified in the configuration file,
            # then we will not make changes
            cur_domains = client.get_app_domains(app_id)
            cur_domains_map = dict()
            for dn in cur_domains:
                cur_domains_map[dn['domain']] = dn

            domain_changed = False
            for dn in domains:
                if dn not in cur_domains_map:
                    # new domain
                    domain_changed = True
                else:
                    del cur_domains_map[dn]

            if cur_domains_map:
                # domain has been removed
                domain_changed = True

            if domain_changed:
                try:
                    app = client.get_app()
                    info(f"updating app domains, app id: {app_id}, file: app.yaml, line: {line(app_config)}")
                    client.put_app(app_id=app_id, domains=domains, label=app.get('name', None))
                    need_release = True
                except Exception as e:
                    clear_changes(client, app_id)
                    error(f"failed to update app domains, app id: {app_id}, file: app.yaml, line: {line(app_config)}, error: {e}")

        if need_release:
            release_changes(client, app_id, staging_release=staging_release)

        ctx['app_id'] = app_id
        return app_id

    staging_release = ctx.get('staging_release', None)
    # app not found, create app
    app_id = add_http_app(client, partition_id, domains, domain, log_format, staging_release)

    ctx['app_id'] = app_id

    return app_id

def cleanup_http_app(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']

    if not domain:
        warn("without specifying a domain name, the application will not be cleaned")

    # delete app
    app_id = get_app_id_by_domain(client, partition_id, domain)
    if app_id:
        info(f"removing app, app id: {app_id}")
        ok = client.del_app(app_id)
        if not ok:
            raise Exception(f"failed to delete app, app id: {app_id}, domain: {domain}")

def export_app_config(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['export_to_path']

    # get app in partition
    app_id = get_app_id_by_domain(client, partition_id, domain)
    if not app_id:
        ctx['app_not_found'] = True
        warn(f"app not found, app id: {app_id}, domain: {domain}")
        return

    ctx['app_id'] = app_id

    export_data = {}

    # process domains
    domains = client.get_app_domains(app_id)
    if domains:
        export_data['domains'] = [domain['domain'] for domain in domains]

    # process access_log_format
    app_config = client.get_app_config(app_id)
    access_log = app_config.get('access_log', {})
    if access_log and 'name' in access_log:
        export_data['access_log_format'] = access_log['name']

    write_yaml_config(configs_path, "app.yaml", export_data)

    info(f"App configuration exported to app.yaml")



# -*- coding: utf-8 -*-
# import io
import sdk_test

class TestLMDB(sdk_test.TestSdk):
    def test_lmdb_backup(self):
        result = self.client.lmdb_backup()
        if result is not True:
            self.assertIs(type(result), dict)
        else:
            self.assertIs(type(result), bool)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
This file is used to release app pending_changes
"""

import argparse

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def main(args):
    app_id = args.app_id

    client = py_client.get_client()
    client.use_app(app_id)

    pending_changes = client.pending_changes()

    if args.dry_run:
        print('app {}: pending_changes: {}'.format(app_id, pending_changes))
        return

    ret = client.new_release()
    if ret:
        print('app {}: release ok: {} changes.'
              .format(str(app_id), pending_changes))
    else:
        print('app {}: release failed'.format(str(app_id)))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='release app pending changes.')

    parser.add_argument('--dry-run',
                        dest='dry_run',
                        action='store_true',
                        help='dry run.')
    parser.add_argument('--app-id',
                        dest='app_id',
                        type=int,
                        required=True,
                        help='specifiy the app_id to operate on.')

    args = parser.parse_args()

    main(args)



# -*- coding: utf-8 -*-

import unittest
import sys
import re


def main(argv):
    suite = unittest.TestSuite()
    testLoader = unittest.TestLoader

    pattern = argv[0] if argv and argv[0] else 'test_*.py'
    mobj = re.match('.*/?tests/(.+)', pattern)
    if mobj:
        pattern = mobj.group(1)

    suite.addTests(testLoader().discover('./tests', pattern=pattern))

    runner = unittest.TextTestRunner(verbosity=2)
    result = runner.run(suite)
    sys.exit(not result.wasSuccessful())


if __name__ == "__main__":
    main(sys.argv[1:])



import os

from .utils import error, warn, info, get_md5_from_comment, \
    cal_config_md5, line, get_real_comment
from .read_config import read_yaml_config, write_yaml_config, add_before_comment

def check_global_basic_auth_groups(groups, filename):
    if not isinstance(groups, list):
        error(f"Unsupported file format for global basic auth groups, file: {filename}")

    for group in groups:
        if 'name' not in group:
            error(f"Global basic auth group name not found, file: {filename}")

        if 'users' not in group:
            error(f"Global basic auth group users not found, file: {filename}")

        label = group.get('label', '')
        group_name = group['name']
        if not isinstance(label, str):
            error(f"Global basic auth group label must be a string, file: {filename}, line: {line(label)}")

        users = group['users']
        for user in users:
            username = user.get("username", None)
            password = user.get("password", None)

            if not isinstance(username, str) or not username:
                error(f"Invalid username in global basic auth group {group_name}: {username}, file: {filename}, line: {line(user)}")

            if not isinstance(password, str) or not password:
                error(f"Invalid password in global basic auth group {group_name}: {password}, file: {filename}, line: {line(user)}")

    return True

def update_global_basic_auth_users(client, group_id, users):
    old_users = client.get_global_basic_auth_users_in_group(group_id)

    old_users_map = {u['username']: u for u in old_users}

    for user in users:
        username = user['username']
        password = user['password']

        if username in old_users_map:
            try:
                info(f"Updating global basic auth user \"{username}\" in group, group id: {group_id}")
                user_id = old_users_map[username]['id']
                client.put_global_basic_auth_user(user_id, group_id, username, password)
            except Exception as e:
                error(f"Failed to update global basic auth user, username: {username}", e)

            del old_users_map[username]
        else:
            try:
                info(f"Adding global basic auth user \"{username}\" to group, group id: {group_id}")
                client.new_global_basic_auth_user(group_id, username, password)
            except Exception as e:
                error(f"Failed to add global basic auth user, username: {username}", e)

    for username, user in old_users_map.items():
        try:
            info(f"Deleting global basic auth user \"{username}\" from group, group id: {group_id}")
            client.del_global_basic_auth_user(user["id"], group_id)
        except Exception as e:
            error(f"Failed to delete global basic auth user, username: {username}", e)

def global_basic_auth_group_changed(old_group, new_md5):
    if 'label' not in old_group:
        return True

    old_md5 = get_md5_from_comment(old_group['label'])
    if new_md5 and old_md5 == new_md5:
        return False

    return True

def process_global_basic_auth_groups(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_basic_auth_groups"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if global basic auth groups are valid")
    for filename, groups in configs.items():
        check_global_basic_auth_groups(groups, filename)

    info("Checking if global basic auth groups have changed")
    old_groups = client.get_all_global_basic_auth_user_groups()
    old_groups_map = {g['name']: g for g in old_groups}

    for filename, new_groups in configs.items():
        for new_group in new_groups:
            name = new_group['name']
            md5 = cal_config_md5(new_group)

            if name not in old_groups_map:
                try:
                    info(f"Adding global basic auth group \"{name}\"")
                    label = new_group.get('label', '')
                    label = f"{label}md5: {md5}, please do not modify."
                    group_id = client.new_global_basic_auth_user_group(name, label=label)
                    update_global_basic_auth_users(client, group_id, new_group['users'])
                except Exception as e:
                    error(f"Failed to add global basic auth group, file: {filename}, line: {line(new_group)}", e)
                continue

            if global_basic_auth_group_changed(old_groups_map[name], md5):
                label = new_group.get('label', '')
                if label:
                    label = f"{label}md5: {md5}, please do not modify."
                else:
                    label = old_groups_map[name].get('label', '')
                    if label:
                        label = get_real_comment(label)
                        label = f"{label}md5: {md5}, please do not modify."
                    else:
                        label = f"md5: {md5}, please do not modify."

                try:
                    group_id = old_groups_map[name]['id']
                    update_global_basic_auth_users(client, group_id, new_group['users'])
                    info(f"Updating global basic auth group \"{name}\"")
                    client.put_global_basic_auth_user_group(group_id, name, label=label)
                except Exception as e:
                    error(f"Failed to update global basic auth group, file: {filename}, line: {line(new_group)}", e)

            del old_groups_map[name]

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # for name, group in old_groups_map.items():
    #     try:
    #         info(f"Deleting global basic auth group \"{name}\"")
    #         client.del_global_basic_auth_user_group(group["id"])
    #     except Exception as e:
    #         error(f"Failed to delete global basic auth group, group id: {group['id']}", e)

def export_global_basic_auth_groups(ctx):
    client = ctx['client']
    configs_path = ctx['export_to_path']
    export_users = ctx.get('export_users', False)

    groups = client.get_all_global_basic_auth_user_groups()
    if not groups:
        info("No global basic auth groups found")
        return

    formatted_groups = []
    for group in groups:
        formatted_group = {
            'name': group['name'],
            'label': get_real_comment(group.get('label', '')),
            'users': []
        }

        if export_users:
            password = ''
            if ctx['export_fake_info']:
                password = "********"

            users = client.get_global_basic_auth_users_in_group(group['id'])
            for user in users:
                formatted_user = {
                    'username': user['username'],
                    'password': password
                }
                formatted_user = add_before_comment(formatted_user, 'username',
                                                       "password: We don't export actual password for security reasons")
                formatted_group['users'].append(formatted_user)

        formatted_groups.append(formatted_group)

    if not export_users:
        warn("global basic auth users will not be exported")

    export_path = os.path.join(configs_path, 'global_basic_auth_groups')

    try:
        write_yaml_config(export_path, "global_basic_auth_groups.yaml", formatted_groups)
        info("Global basic auth groups exported successfully to global_basic_auth_groups/global_basic_auth_groups.yaml")
    except Exception as e:
        error("Failed to export global basic auth groups to global_basic_auth_groups/global_basic_auth_groups.yaml", e)

def cleanup_global_basic_auth_groups(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']

    # groups = client.get_all_global_basic_auth_user_groups()

    # for group in groups:
    #     try:
    #         info(f"Removing global basic auth group \"{group['name']}\"")
    #         client.del_global_basic_auth_user_group(group["id"])
    #     except Exception as e:
    #         error(f"Failed to remove global basic auth group, group id: {group['id']}", e)



import os
import ipaddress

from .utils import error, warn, info, line
from .read_config import read_yaml_config, write_yaml_config
from .ip_lists import check_ip_lists, is_change_ip_list

def process_global_ip_lists(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_ip_lists"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if IP lists are valid")
    for filename, ip_lists in configs.items():
        check_ip_lists(ip_lists, filename)

    old_ip_lists = client.get_all_global_ip_lists()

    old_ip_lists_dict = {ip_list['name']: ip_list for ip_list in old_ip_lists}

    info("Checking if IP lists have changed")
    new_ip_lists_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        ip_lists = configs[filename]
        for ip_list in ip_lists:
            new_ip_lists_dict[ip_list['name']] = True

            if ip_list['name'] in old_ip_lists_dict:
                old_ip_list = old_ip_lists_dict[ip_list['name']]
                if is_change_ip_list(ip_list, old_ip_list):
                    try:
                        info(f"Updating Global IP list \"{ip_list['name']}\"")
                        client.put_global_ip_list(
                            rule_id=old_ip_list['id'],
                            name=ip_list['name'],
                            type=ip_list['type'],
                            items=ip_list['items']
                        )
                    except Exception as e:
                        error(f"Failed to update IP list, file: {filename}, line: {line(ip_list)}", e)
            else:
                try:
                    info(f"Adding global IP list \"{ip_list['name']}\"")
                    client.new_global_ip_list(
                        name=ip_list['name'],
                        type=ip_list['type'],
                        items=ip_list['items']
                    )
                except Exception as e:
                    error(f"Failed to add IP list to app, file: {filename}, line: {line(ip_list)}", e)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # for list_name, ip_list in old_ip_lists_dict.items():
    #     if list_name not in new_ip_lists_dict:
    #         try:
    #             info(f"Removing Global IP list \"{list_name}\"")
    #             client.del_global_ip_list(ip_list['id'])
    #         except Exception as e:
    #             error(f"Failed to remove Global IP list,list id: {ip_list['id']}", e)

def cleanup_global_ip_lists(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']
    # partition_id = ctx['partition_id']
    # domain = ctx['domain']

    # ip_lists = client.get_all_global_ip_lists()

    # for ip_list in ip_lists:
    #     try:
    #         info(f"Removing Global IP list \"{ip_list['name']}\"")
    #         client.del_global_ip_list(ip_list['id'])
    #     except Exception as e:
    #         error(f"Failed to remove Global IP list from app, list id: {ip_list['id']}", e)

def export_global_ip_lists(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['export_to_path']

    ip_lists = client.get_all_global_ip_lists()

    if not ip_lists:
        info(f"No IP Global lists found")
        return

    formatted_ip_lists = []
    for ip_list in ip_lists:
        items = ip_list['items']
        new_items = list()
        for item in items:
            new_items.append({'ip': item['ip']})

        formatted_list = {
            'name': ip_list['name'],
            'type': ip_list['type'],
            'items': new_items
        }
        formatted_ip_lists.append(formatted_list)

    export_path = os.path.join(configs_path, "global_ip_lists")

    try:
        write_yaml_config(export_path, "global_ip_lists.yaml", formatted_ip_lists)
        info(f"Global IP lists exported successfully to global_ip_lists/global_ip_lists.yaml")
    except Exception as e:
        error(f"Failed to export IP lists to global_ip_lists/global_ip_lists.yaml", e)



# -*- coding: utf-8 -*-
# import io
import sdk_test
import string
import random

class TestParitionLuaModule(sdk_test.TestSdk):
    def test_partition_lua_module(self):
        data = self.client.get_all_partition_lua_module(1)
        self.assertIs(type(data), list)
        self.assertTrue(len(data) >= 0)

        data = self.client.get_partition_lua_module(1, 1000000)
        self.assertEqual(data, {})

        code = """
        local _M = {}

        _M.test = 1

        return _M
        """
        pname = ''.join(random.choices(string.ascii_uppercase + string.digits, k = 6))
        name = "test" + pname
        data = self.client.new_partition_lua_module(1, name, code)
        id = data["id"]
        self.assertIs(type(id), int)

        data = self.client.get_all_partition_lua_module(1)
        self.assertTrue(len(data) > 0)

        new_name = "test2" + pname
        data = self.client.put_partition_lua_module(1, id, new_name)
        self.assertEqual(data, True)

        new_code = """
        local _M = {}

        _M.test = 2

        return _M
        """
        data = self.client.put_partition_lua_module(1, id, None, new_code)
        self.assertEqual(data, True)

        data = self.client.get_partition_lua_module(1, id)
        self.assertEqual(data["id"], id)
        self.assertEqual(data["name"], new_name)
        self.assertEqual(data["code"], new_code)

        data = self.client.del_partition_lua_module(1, id)
        self.assertEqual(data, True)



# -*- coding: utf-8 -*-
# import io
import sdk_test


class TestGlobalK8S(sdk_test.TestSdk):
    def test_global_k8s(self):
        client = self.client

        k8s_id = client.new_global_k8s(name = "k8s_sdk_test", host = "120.24.93.4",
            port = 9443, token = "eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJkZWZhdWx0Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZWNyZXQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5LXRva2VuLTg1YjVkIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiZGRkZjRmODItODcyZi00ODRiLWI1ZTgtYjEwMWZjYzI5ODcxIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50OmRlZmF1bHQ6b3BlbnJlc3R5LWVkZ2Utd3kifQ.WldqXAWALlSJlo9lY6wsES4iE-BJcEnDSK_YD4M2xMx8qYvrzCdzjcRX6Dl56tR1Yv98QutEcyG_hjfjdYt_pDI_pgni3f4-jEjXrSDOiPw7kXschhaqVaN1mDEOGuOi2mtGtfEz6tPg-j4dbUuvmR3S5niBAh9tbe9_IacRiEt9HEYw0W-knoHyrOofHUY99JCnVjtXADhfLMQQyo0Ok8FNEjb06qMWVF_vwy6CJKWZy6dmVX1SXT494rToy97dEvfHtMqSxQOnCEKZ-8XwxgUTBPExpbX52tyJ8D2xmIorQVnJovCL2anwLMieYkD5F11KtFDozI510AyXR2i9FA", ssl_verify = False)

        self.assertIs(type(k8s_id), int)
        self.assertGreater(k8s_id, 0)
        self.global_k8s_id = k8s_id

        data = client.get_global_k8s(k8s_id)

        self.assertEqual(data["name"], "k8s_sdk_test")
        self.assertEqual(data["host"], "120.24.93.4")
        self.assertEqual(data["port"], 9443)
        self.assertEqual(data["id"], k8s_id)
        self.assertEqual(data["ssl_verify"], False)

        all_k8s = client.get_all_global_k8s()

        self.assertEqual(len(all_k8s), 1)

        self.assertEqual(all_k8s[0]["id"], k8s_id)

        # no k8s enviroment in ci
        k8s_detail = client.get_k8s_services_detail(k8s_id)
        self.assertNotEqual(k8s_detail, None)

        k8s_services = [
            {
                'k8s' : k8s_id,
                'k8s_namespace' : 'default',
                'k8s_service' : 'test-hello',
                'k8s_service_port' : 80,
            }
        ]

        nodes = [
            {
                'ip' : '172.20.2.139',
                'port' : 80,
                'weight' : 3,
            }
        ]

        upstream_id = client.new_global_k8s_upstream(name = 'test_global_k8s_sdk', k8s_services = k8s_services, nodes = nodes)

        # no k8s environment in ci
        res = client.put_global_k8s_upstream(up_id = upstream_id, name = "test_global_k8s_sdk2", k8s_services = k8s_services)
        self.assertEqual(res, True)

        res = client.get_global_k8s_upstream(upstream_id)
        self.assertEqual(res['k8s_services'][0]['k8s'], k8s_id)
        self.assertEqual(res['k8s_services'][0]['k8s_namespace'], 'default')
        self.assertEqual(res['k8s_services'][0]['k8s_service'], 'test-hello')
        self.assertEqual(res['k8s_services'][0]['k8s_service_port'], 80)

        res = client.get_all_global_k8s_upstreams()
        self.assertEqual(len(res), 1)

        res = client.del_global_k8s_upstream(upstream_id)
        self.assertEqual(res, True)



import os
import sys

from py_client import client

print('delete all apps')
apps = client.get_all_apps()
for app_id in apps:
    ok = client.del_app(app_id)
    if ok:
        print('deleted app: ' + str(app_id))
    else:
        print('failed to delete app: ' + str(app_id))

print('done')




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from os import getcwd, path, system, remove, walk
import sys

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)

IMPORT_CODE = 'import py_client\nclient = py_client.get_client()\n'
COMMON_TLDS = ['com', 'net', 'cn', 'org', 'edu']


def gen_n_run(name, root):
    app_file = path.join(root, name)
    with open(app_file) as file_handler:
        content = file_handler.read()
        tmp_file = path.join(path.dirname(__file__), name)
        with open(tmp_file, 'w') as file_writer:
            file_writer.write(IMPORT_CODE)
            file_writer.write(content)

        print('\n-------------- init ------------')
        print('copy <{}> to <{}>'.format(app_file, tmp_file))

        n = system(sys.executable + ' ' + tmp_file)
        if n == 0:
            print('created app with ' + name)
        remove(tmp_file)


def main():
    global_config_file_path = path.join(getcwd(), "global.ini")
    if not path.isfile(global_config_file_path):
        print('ERROR: global.ini: No such File or Directory')
        return 1

    apps_dir = py_client.get_apps_dir()

    if len(sys.argv) > 1 and sys.argv[1]:
        apps_dir = sys.argv[1]

    if not path.isdir(apps_dir):
        return

    for root, dirs, files in walk(apps_dir):
        for name in files:
            gen_n_run(name, root)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# -*- coding: UTF-8 -*-
"""
This file is used to download emergency nginx.conf from admin server
"""

from urllib.parse import urljoin
from os import environ
import io
import warnings
import requests

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)

NODE_SERVER = environ.get("NODE_SERVER") \
        or "http://127.0.0.1:8091"
VER_URL = urljoin(NODE_SERVER, "/version")

EMER_PATH = environ.get("EMER_PATH") \
        or "/usr/local/oredge-node/conf/emergency.nginx.conf"


def do_node_server(url):
    res = requests.request("GET", url, timeout=30)

    response = res.json()
    return response


def main():
    data = do_node_server(VER_URL)

    partition = data.get('partition')
    if not partition:
        warnings.warn("no partition found.")
        exit(1)

    client = py_client.get_client()
    data = client.emergency_conf(partition)
    if not isinstance(data, dict):
        print("no ngx_conf found.")
        exit(2)

    ngx_conf = data.get('ngx_conf')
    if not ngx_conf:
        print("no ngx_conf found.")
        exit(3)

    with io.open(EMER_PATH, 'w') as file:
        file.write(ngx_conf)
        print("write to ", EMER_PATH, "... ok.")


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-


def proc_cond(conds):
    if conds is None:
        return None

    conditions = []

    for cond in conds:
        condition = {}

        vals = []
        for value in cond.get('values', []):
            val = [value.get('val'), value.get('type')]
            vals.append(val)

        condition['vals'] = vals

        if cond.get('variable'):
            variable = cond.get('variable')

            if variable.get('global_var'):
                condition['global_var'] = variable.get('global_var')

            if variable.get('user_var'):
                condition['user_var'] = variable.get('user_var')

            if variable.get('name'):
                if variable.get('args'):
                    condition['var'] = [
                        variable.get('name'), variable.get('args')
                    ]
                else:
                    condition['var'] = variable.get('name')

        if cond.get('operator'):
            condition['op'] = cond.get('operator').get('name')
        if cond.get('caseless'):
            condition['caseless'] = cond.get('caseless')

        conditions.append(condition)

    return conditions


def proc_conseq(conseq):
    if conseq is None:
        return None

    conseqs = []

    for c in conseq:
        spec = {}

        if c.get('global_action_id'):
            spec['global_action'] = c.get('global_action_id')
        else:
            spec[c.get('type')] = c.get(c.get('type'), {})

        conseqs.append(spec)

    return conseqs


# rule_sets, action, threshold, clearance, redirect_url
def proc_waf(waf):
    if waf is None:
        return waf

    waf_rule = {}

    waf_rule['rule_sets'] = waf.get('rule_sets', [])

    if waf.get('threshold_score') >= 1000:
        waf_rule['threshold'] = 'high'
    elif waf.get('threshold_score') >= 100:
        waf_rule['threshold'] = 'medium'
    else:
        waf_rule['threshold'] = 'low'

    action = waf.get('action')

    if action == 'block':
        waf_rule['action'] = '403 Forbidden'
    else:
        waf_rule['action'] = action

    if action == 'edge-captcha' and waf.get('clearance'):
        waf_rule['clearance_time'] = waf.get('clearance')

    elif action == 'redirect' and waf.get('redirect_url'):
        waf_rule['redirect_url'] = waf.get('redirect_url')

    return waf_rule


def proc_proxy(proxy):
    if proxy is None:
        return None

    upstreams = []
    for upstream in proxy.get('upstream', []):
        upstreams.append({
            'upstream': upstream.get('cluster', None),
            'global_upstream': upstream.get('global_cluster', None),
            'weight': upstream.get('weight')
        })

    backup_upstreams = []
    for upstream in proxy.get('backup_upstream', []):
        backup_upstreams.append({
            'upstream': upstream.get('cluster', None),
            'global_upstream': upstream.get('global_cluster', None),
            'weight': upstream.get('weight')
        })

    balancer_vars = None
    balancer_algorithm = None
    if proxy.get('balancer'):
        balancer_vars = proxy.get('balancer').get('variables')
        balancer_algorithm = proxy.get('balancer').get('algorithm')

    data = {
        'upstreams': upstreams,
        'connect_timeout': proxy.get('connect_timeout'),
        'read_timeout': proxy.get('read_timeout'),
        'send_timeout': proxy.get('send_timeout'),
        'retries': proxy.get('retries'),
        'retry_condition': proxy.get('retry_condition'),
        'multi_tier': proxy.get('layer_policty'),
    }

    if backup_upstreams:
        data['backup_upstreams'] = backup_upstreams

    if balancer_algorithm is not None:
        data['balancer_algorithm'] = balancer_algorithm

        if balancer_vars is not None:
            data['balancer_vars'] = balancer_vars

    return data


# cache_key, enable_default_ttl, default_ttl, default_ttl_unit, enforce_cache
def proc_cache(cache_rule):
    if cache_rule is None:
        return None

    cache_keys = []
    for cache_key in cache_rule.get('cache_key'):
        cache_keys.append(cache_key.get('name'))

    data = {
        'cache_key': cache_keys,
        'enable_default_ttl': cache_rule.get('enable_default_ttl', False),
        'default_ttl': cache_rule.get('default_ttl', 1),
        'default_ttl_unit': cache_rule.get('default_ttl_unit', 'min'),
    }

    if cache_rule.get('enforce_cache'):
        data['enforce_cache'] = cache_rule.get('enforce_cache')

    return data


def proc_content(content_rule):
    if content_rule is None:
        return None

    content = {}

    if content_rule.get('favicon'):
        content['favicon'] = content_rule.get('favicon')

    if content_rule.get('empty_gif'):
        content['empty_gif'] = content_rule.get('empty_gif')

    return content



# -*- coding: utf-8 -*-
# import io
import sdk_test


class TestGateWay(sdk_test.TestSdk):
    def test_get_all_gateway(self):
        tags = self.client.get_all_gateway_tag()

        for tag in tags:
            self.client.del_gateway_tag(tag['id'])

        data=self.client.get_all_gateway()

        tag1 = self.client.add_gateway_tag("tag1")
        tag2 = self.client.add_gateway_tag("tag2")

        self.client.del_gateway_tag(tag1)

        gateway1 = self.client.add_gateway("gateway1", 1, [ tag2 ])

        data=self.client.get_all_gateway()
        self.assertEqual(type(data), type(list()))

        self.client.del_gateway(gateway1)
        self.client.del_gateway_tag(tag2)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
update rule condition values.
"""

import json
import argparse

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def main(args):
    client = py_client.get_client()
    client.use_app(args.app_id)

    rules = client.get_all_rules()
    for rule in rules:
        print('{} rule_id: {} {}'.format('-' * 10, rule.get('id'), '-' * 10))
        print(json.dumps(rule))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='query rule_id by app_id')

    parser.add_argument('--app-id', type=int, dest='app_id',
                        help='Application id', required=True)

    args = parser.parse_args()

    main(args)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
This file is used to update proxy timeout & balancer
"""

import time

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def main():
    client = py_client.get_client()
    app_ids = client.get_all_apps(detail=False)

    for app_id in app_ids:
        client.use_app(app_id)
        rules = client.get_all_rules()
        changed = 0

        for rule in rules:
            if 'proxy' not in rule:
                continue

            proxy = rule['proxy']

            need_update = False

            if proxy['connect_timeout'] == 6 \
                    and proxy['connect_timeout_unit'] == 's' \
                    and proxy['read_timeout'] == 6 \
                    and proxy['read_timeout_unit'] == 's' \
                    and proxy['send_timeout'] == 6 \
                    and proxy['send_timeout_unit'] == 's':
                proxy['connect_timeout'] = 30
                proxy['read_timeout'] = 30
                proxy['send_timeout'] = 30

                need_update = True

            if proxy['balancer'] \
                    and proxy['balancer']['algorithm'] == 'roundrobin':
                proxy['balancer_algorithm'] = 'chash'
                proxy['balancer_vars'] = [{'name': 'client-addr'}]

                need_update = True

            if need_update:
                proxy_upstreams = []
                for upstream in proxy['upstream']:
                    proxy_upstreams.append({
                        'upstream': upstream['cluster'],
                        'weight': upstream.get('weight', 1),
                        })

                proxy_backup_upstreams = []
                if 'backup_upstream' in proxy:
                    for upstream in proxy['upstream']:
                        proxy_backup_upstreams.append({'upstream':
                                                       upstream['cluster']})

                proxy_rule = {
                    'upstreams': proxy_upstreams,
                    'backup_upstreams': proxy_backup_upstreams,
                    'retries': proxy.get('retries', 3),
                    'connect_timeout': proxy.get('connect_timeout', 30),
                    'read_timeout': proxy.get('read_timeout', 30),
                    'send_timeout': proxy.get('send_timeout', 30),
                    'retry_condition': proxy.get('retry_condition', None),
                    'balancer_algorithm': proxy.get('balancer_algorithm',
                                                    'roundrobin'),
                    'balancer_vars': proxy.get('balancer_vars', None),
                }

                ok = client.put_rule(rule_id=rule['id'], proxy=proxy_rule)

                if not ok:
                    print("ERROR: failed to update rule, app_id: "
                          + str(app_id))
                else:
                    changed = changed + 1

            time.sleep(0.1)  # for each rule

        if changed > 0:
            print("changed " + str(changed)
                  + " rules for app, id: " + str(app_id))

        time.sleep(0.5)  # for each application


if __name__ == '__main__':
    main()



import os
import ipaddress

from .utils import error, warn, info, get_app_id_by_domain, release_changes, \
    clear_changes, line
from .read_config import read_yaml_config, write_yaml_config

def check_ip_lists(ip_lists, filename):
    if not isinstance(ip_lists, list):
        error(f"Unsupported IP lists file format, file: {filename}")

    valid_types = ['ipv4']  # 'ipv6' not support yet
    names = dict()

    for ip_list in ip_lists:
        if not isinstance(ip_list, dict):
            error(f"Each IP list must be a dictionary, file: {filename}, line: {line(ip_list)}")

        if 'name' not in ip_list:
            error(f"Missing 'name' in IP list, file: {filename}, line: {line(ip_list)}")

        if 'type' not in ip_list:
            error(f"Missing 'type' in IP list {ip_list['name']}, file: {filename}, line: {line(ip_list)}")

        if ip_list['type'] not in valid_types:
            error(f"Invalid type '{ip_list['type']}' for IP list {ip_list['name']}, file: {filename}, line: {line(ip_list)}")

        if 'items' not in ip_list or not isinstance(ip_list['items'], list):
            error(f"Missing or invalid 'items' for IP list {ip_list['name']}, file: {filename}, line: {line(ip_list)}")

        for item in ip_list['items']:
            if not isinstance(item, dict) or 'ip' not in item:
                error(f"Invalid item format in IP list {ip_list['name']}, file: {filename}, line: {line(item)}")
            try:
                if ip_list['type'] == 'ipv4':
                    if '/' in item['ip']:
                        ipaddress.IPv4Network(item['ip'], strict=False)
                    else:
                        ipaddress.IPv4Address(item['ip'])
                else:
                    if '/' in item['ip']:
                        ipaddress.IPv6Network(item['ip'], strict=False)
                    else:
                        ipaddress.IPv6Address(item['ip'])
            except ValueError:
                error(f"Invalid IP address '{item['ip']}' in IP list {ip_list['name']}, file: {filename}, line: {line(item)}")

        if ip_list['name'] in names:
            error(f"Duplicate name in IP list {ip_list['name']}, file: {filename}, line: {line(ip_list)}")

        names[ip_list['name']] = True

    return True

def is_change_ip_list(new_list, old_list):
    if (new_list['name'] != old_list['name'] or
            new_list['type'] != old_list['type']):
        return True

    new_ips = set(item['ip'] for item in new_list['items'])
    old_ips = set(item['ip'] for item in old_list['items'])
    return new_ips != old_ips

def process_ip_lists(ctx):
    client = ctx['client']
    app_id = ctx['app_id']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)
    staging_release = ctx.get('staging_release', None)

    if not location:
        location = "ip_lists"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if IP lists are valid")
    for filename, ip_lists in configs.items():
        check_ip_lists(ip_lists, filename)

    client.use_app(app_id)
    old_ip_lists = client.get_all_ip_lists()

    old_ip_lists_dict = {ip_list['name']: ip_list for ip_list in old_ip_lists}

    info("Checking if IP lists have changed")
    new_ip_lists_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        ip_lists = configs[filename]
        for ip_list in ip_lists:
            new_ip_lists_dict[ip_list['name']] = True

            if ip_list['name'] in old_ip_lists_dict:
                old_ip_list = old_ip_lists_dict[ip_list['name']]
                if is_change_ip_list(ip_list, old_ip_list):
                    try:
                        info(f"Updating IP list \"{ip_list['name']}\", app id: {app_id}")
                        client.put_ip_list(
                            rule_id=old_ip_list['id'],
                            name=ip_list['name'],
                            type=ip_list['type'],
                            items=ip_list['items']
                        )
                    except Exception as e:
                        clear_changes(client, app_id)
                        error(f"Failed to update IP list, file: {filename}, line: {line(ip_list)}", e)
            else:
                try:
                    info(f"Adding IP list \"{ip_list['name']}\" to app, app id: {app_id}")
                    client.new_ip_list(
                        name=ip_list['name'],
                        type=ip_list['type'],
                        items=ip_list['items']
                    )
                except Exception as e:
                    clear_changes(client, app_id)
                    error(f"Failed to add IP list to app, file: {filename}, line: {line(ip_list)}", e)

    for list_name, ip_list in old_ip_lists_dict.items():
        if list_name not in new_ip_lists_dict:
            try:
                info(f"Removing IP list \"{list_name}\" from app, app id: {app_id}")
                client.del_ip_list(ip_list['id'])
            except Exception as e:
                clear_changes(client, app_id)
                error(f"Failed to remove IP list from app, app id: {app_id}, list id: {ip_list['id']}", e)

    release_changes(client, app_id, staging_release=staging_release)

def cleanup_ip_lists(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    staging_release = ctx.get('staging_release', None)

    app_id = get_app_id_by_domain(client, partition_id, domain)
    if not app_id:
        error(f"App not found, app id: {app_id}, domain: {domain}")

    client.use_app(app_id)
    ip_lists = client.get_all_ip_lists()

    for ip_list in ip_lists:
        try:
            info(f"Removing IP list \"{ip_list['name']}\" from app, app id: {app_id}")
            client.del_ip_list(ip_list['id'])
        except Exception as e:
            clear_changes(client, app_id)
            error(f"Failed to remove IP list from app, app id: {app_id}, list id: {ip_list['id']}", e)

    release_changes(client, app_id, staging_release=staging_release)

def export_ip_lists(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['export_to_path']

    if ctx.get('app_not_found', False) is True:
        return

    app_id = ctx.get('app_id', None)
    if app_id is None:
        app_id = get_app_id_by_domain(client, partition_id, domain)
        if not app_id:
            warn(f"App not found, partition_id: {partition_id}, domain: {domain}")
            return

        ctx['app_id'] = app_id

    client.use_app(app_id)
    ip_lists = client.get_all_ip_lists()

    if not ip_lists:
        info(f"No IP lists found for app_id: {app_id}")
        return

    formatted_ip_lists = []
    for ip_list in ip_lists:
        items = ip_list['items']
        new_items = list()
        for item in items:
            new_items.append({'ip': item['ip']})

        formatted_list = {
            'name': ip_list['name'],
            'type': ip_list['type'],
            'items': new_items
        }
        formatted_ip_lists.append(formatted_list)

    export_path = os.path.join(configs_path, "ip_lists")

    try:
        write_yaml_config(export_path, "ip_lists.yaml", formatted_ip_lists)
        info(f"IP lists exported successfully to ip_lists/ip_lists.yaml")
    except Exception as e:
        error(f"Failed to export IP lists to ip_lists/ip_lists.yaml", e)



# -*- coding: utf-8 -*-
# import io
import sdk_test


class TestK8sUpstream(sdk_test.TestSdk):
    def test_k8s_upstream(self):
        client = self.client
        k8s_id = client.new_global_k8s(name = "k8s_sdk_test", host = "192.168.122.220",
            port = 9443, token = "eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJkZWZhdWx0Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZWNyZXQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5LXRva2VuLTg1YjVkIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiZGRkZjRmODItODcyZi00ODRiLWI1ZTgtYjEwMWZjYzI5ODcxIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50OmRlZmF1bHQ6b3BlbnJlc3R5LWVkZ2Utd3kifQ.WldqXAWALlSJlo9lY6wsES4iE-BJcEnDSK_YD4M2xMx8qYvrzCdzjcRX6Dl56tR1Yv98QutEcyG_hjfjdYt_pDI_pgni3f4-jEjXrSDOiPw7kXschhaqVaN1mDEOGuOi2mtGtfEz6tPg-j4dbUuvmR3S5niBAh9tbe9_IacRiEt9HEYw0W-knoHyrOofHUY99JCnVjtXADhfLMQQyo0Ok8FNEjb06qMWVF_vwy6CJKWZy6dmVX1SXT494rToy97dEvfHtMqSxQOnCEKZ-8XwxgUTBPExpbX52tyJ8D2xmIorQVnJovCL2anwLMieYkD5F11KtFDozI510AyXR2i9FA", ssl_verify = False)

        self.assertIs(type(k8s_id), int)
        self.assertGreater(k8s_id, 0)
        self.global_k8s_id = k8s_id

        k8s_services = [
            {
                'k8s' : k8s_id,
                'k8s_namespace' : 'default',
                'k8s_service' : 'test-hello',
                'k8s_service_port' : 80,
            }
        ]

        nodes = [
            {
                'ip' : "172.20.2.139",
                'port' : 80,
                'weight' : 3,
            }
        ]

        upstream_id = client.new_k8s_upstream(name = 'test_k8s_sdk', k8s_services = k8s_services, nodes = nodes)

        # no k8s environment in ci
        res = client.put_k8s_upstream(up_id = upstream_id, name = "test_k8s_sdk2", k8s_services = k8s_services)
        self.assertEqual(res, True)

        res = client.get_k8s_upstream(upstream_id)
        self.assertEqual(res['k8s_services'][0]['k8s'], k8s_id)
        self.assertEqual(res['k8s_services'][0]['k8s_namespace'], 'default')
        self.assertEqual(res['k8s_services'][0]['k8s_service'], 'test-hello')
        self.assertEqual(res['k8s_services'][0]['k8s_service_port'], 80)

        res = client.get_all_k8s_upstreams()
        self.assertEqual(len(res), 1)


        normal_up_id = client.new_upstream(
            name='origin-upstream',
            servers=[
                {'ip': '172.22.31.1', 'port': 80},
                {'ip': '172.22.31.2', 'port': 80, 'weight': 2}
            ])

        normal_upstream = client.get_upstream(normal_up_id)

        proxy_rule = {
            'upstreams': [{'k8s_upstream': upstream_id, 'weight': 1}, {'upstream': normal_up_id}],
            'retries': 2,
            'retry_condition' : ["invalid_header", "http_500"],
            'balancer_algorithm' : "roundrobin",
        }

        rule_id = client.new_rule(proxy = proxy_rule)
        self.assertNotEqual(rule_id, None)

        copy_up_id = client.copy_upstream_to_k8s_upstream(normal_up_id, k8s_services=k8s_services)

        self.assertIs(type(copy_up_id), int)

        copy_upstream = client.get_k8s_upstream(copy_up_id)


        self.assertEqual(copy_upstream.get('name'), normal_upstream.get('name'))
        self.assertEqual(copy_upstream.get('ssl'), normal_upstream.get('ssl'))
        self.assertEqual(copy_upstream.get('disable_ssl_verify'), normal_upstream.get('disable_ssl_verify'))
        self.assertEqual(copy_upstream.get('enable_checker'), normal_upstream.get('enable_checker'))

        res = client.get_rule(rule_id)

        proxy = res.get('proxy')
        self.assertNotEqual(proxy, None)

        upstreams = proxy.get('upstream')
        self.assertNotEqual(upstreams, None)

        for up in upstreams:
            k8s_cluster = up.get('k8s_cluster')
            self.assertNotEqual(k8s_cluster, None)

        res = client.del_rule(rule_id)
        self.assertEqual(res, True)

        res = client.del_k8s_upstream(copy_up_id)
        self.assertEqual(res, True)

        res = client.del_k8s_upstream(upstream_id)
        self.assertEqual(res, True)

    def test_search_k8s_upstream(self):
        client = self.client

        k8s_id = client.new_global_k8s(name = "k8s_sdk_test", host = "120.24.93.4",
            port = 9443, token = "eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJkZWZhdWx0Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZWNyZXQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5LXRva2VuLTg1YjVkIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQubmFtZSI6Im9wZW5yZXN0eS1lZGdlLXd5Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiZGRkZjRmODItODcyZi00ODRiLWI1ZTgtYjEwMWZjYzI5ODcxIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50OmRlZmF1bHQ6b3BlbnJlc3R5LWVkZ2Utd3kifQ.WldqXAWALlSJlo9lY6wsES4iE-BJcEnDSK_YD4M2xMx8qYvrzCdzjcRX6Dl56tR1Yv98QutEcyG_hjfjdYt_pDI_pgni3f4-jEjXrSDOiPw7kXschhaqVaN1mDEOGuOi2mtGtfEz6tPg-j4dbUuvmR3S5niBAh9tbe9_IacRiEt9HEYw0W-knoHyrOofHUY99JCnVjtXADhfLMQQyo0Ok8FNEjb06qMWVF_vwy6CJKWZy6dmVX1SXT494rToy97dEvfHtMqSxQOnCEKZ-8XwxgUTBPExpbX52tyJ8D2xmIorQVnJovCL2anwLMieYkD5F11KtFDozI510AyXR2i9FA", ssl_verify = False)

        self.assertIs(type(k8s_id), int)
        self.assertGreater(k8s_id, 0)
        self.global_k8s_id = k8s_id

        k8s_services = [
            {
                'k8s' : k8s_id,
                'k8s_namespace' : 'default',
                'k8s_service' : 'test-hello',
                'k8s_service_port' : 80,
            }
        ]

        k8s_up_id = self.client.new_k8s_upstream(name = 'test_search_k8s_upstream1', k8s_services = k8s_services)
        k8s_global_up_id = self.client.new_global_k8s_upstream(name = 'test_search_k8s_upstream2', k8s_services = k8s_services)

        up_id = self.client.new_upstream(
            name='test_search_k8s_upstream3',
            servers=[
                {'ip': '172.22.31.1', 'port': 80},
                {'ip': '172.22.31.2', 'port': 80, 'weight': 2}
            ])

        global_up_id = self.client.new_global_upstream(
            name = 'test_search_k8s_upstream4',
            servers=[
                {'ip': '172.22.31.1', 'port': 80},
                {'ip': '172.22.31.2', 'port': 80, 'weight': 2}
            ])


        data = self.client.search_k8s_upstream_by_name("k8s_upstream")
        self.assertEqual(len(data), 2)

        for d in data:
            if d['name'] == "test_search_k8s_upstream1":
                self.assertEqual(d['upstream_type'], "k8s_http")
            elif d['name'] == "test_search_k8s_upstream2":
                self.assertEqual(d['upstream_type'], "k8s_global")
            else:
                raise Exception("search wrong k8s upstream")

        data = self.client.search_upstream_by_name("k8s_upstream")
        self.assertEqual(len(data), 2)

        for d in data:
            if d['name'] == "test_search_k8s_upstream3":
                self.assertEqual(d['upstream_type'], "http")
            elif d['name'] == "test_search_k8s_upstream4":
                self.assertEqual(d['upstream_type'], "global")
            else:
                raise Exception("search wrong not k8s upstream")

        data = self.client.search_k8s_upstream(namespace = "default", type_list=["k8s_global"])

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['name'], 'test_search_k8s_upstream2')

        app_id = self.client.get_app_id()

        data = self.client.search_k8s_upstream(namespace = "default", type_list=["k8s_http"])

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['name'], 'test_search_k8s_upstream1')
        self.assertEqual(data[0]['id'], k8s_up_id)
        self.assertEqual(data[0]['app_id'], app_id)

        data = self.client.search_k8s_upstream(service = "test-hello", type_list=["k8s_http"])

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['name'], 'test_search_k8s_upstream1')
        self.assertEqual(data[0]['id'], k8s_up_id)
        self.assertEqual(data[0]['app_id'], app_id)

        data = self.client.search_k8s_upstream(port = 80, type_list=["k8s_http"])

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['name'], 'test_search_k8s_upstream1')
        self.assertEqual(data[0]['id'], k8s_up_id)
        self.assertEqual(data[0]['app_id'], app_id)

        data = self.client.search_k8s_upstream(port = 80, service = "test-hello", type_list=["k8s_http"])

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['name'], 'test_search_k8s_upstream1')
        self.assertEqual(data[0]['id'], k8s_up_id)
        self.assertEqual(data[0]['app_id'], app_id)

        data = self.client.search_k8s_upstream(port = 80, service = "test-hello2", type_list=["k8s_http"])

        self.assertEqual(len(data), 0)

        data = self.client.search_k8s_upstream_history(page=1, pagesize=5)

        self.assertEqual(len(data), 5)

        data = self.client.get_k8s_upstream(k8s_up_id)
        self.assertNotEqual(data, None)

        ip = data['nodes'][0]['ip']

        data = self.client.search_k8s_upstream_by_ip(ip = ip)
        self.assertEqual(len(data), 2)

        self.client.del_k8s_upstream(k8s_up_id)
        self.client.del_global_k8s_upstream(k8s_global_up_id)
        self.client.del_upstream(up_id)
        self.client.del_global_upstream(global_up_id)



# -*- coding: utf-8 -*-
# import io
import sdk_test

class TestReferenced(sdk_test.TestSdk):
    def test_get_global_cert_referenced(self):
        data=self.client.get_global_cert_referenced(1)
        self.assertIs(type(data), list)
        self.assertTrue(len(data) >= 0)



import os
import smtplib
from email.mime.text import MIMEText
from email.header import Header

from .utils import error, warn, info
from .read_config import read_yaml_file

def read_email_configs(config_file):
    if config_file is None:
        return None

    if not os.path.exists(config_file):
        return None

    configs = read_yaml_file(config_file)
    if configs is None:
        return None

    if 'sender' not in configs:
        error(f"sender not found in email config file: {config_file}")

    if 'password' not in configs:
        error(f"sender password not found in email config file: {config_file}")

    if 'new_user_email_template' not in configs:
        error(f"email template not found in email config file: {config_file}")

    email_template = configs.get('new_user_email_template', None)
    if 'subject' not in email_template:
        error(f"email template subject not found in email config file: {config_file}")

    if 'content' not in email_template:
        error(f"email template content not found in email config file: {config_file}")

    if 'smtp_server' not in configs:
        error(f"smtp server not found in email config file: {config_file}")

    if 'smtp_port' not in configs:
        warn(f"smtp port not found in email config file: {config_file}, using default port: 587")

    return configs

def send_email(configs, receiver, subject, content):
    sender = configs['sender']
    password = configs['password']
    smtp_server = configs['smtp_server']
    smtp_port = configs.get('smtp_port', 587)

    message = MIMEText(content, 'plain', 'utf-8')
    message['Subject'] = Header(subject, 'utf-8')

    res = False
    try:
        smtp = smtplib.SMTP(smtp_server, smtp_port)
        smtp.starttls()

        smtp.login(sender, password)

        smtp.sendmail(sender, receiver, message.as_string())
        info("send email to " + receiver + " success")
        res = True
    except smtplib.SMTPException as e:
        warn("send email to " + receiver + " failed: " + str(e))
    finally:
        smtp.quit()

    return res



# -*- coding: utf-8 -*-

import warnings
import os
import sys
import hashlib
import requests
import argparse
import urllib3
import datetime
import traceback
import socket
from edge2client import Edge2Client
from .utils import error, warn, info, md5sum, cal_config_md5, \
    release_changes, clear_changes, release_partition_changes, \
    clear_partition_changes, line
from .http_app import process_http_app, cleanup_http_app, export_app_config
from .read_config import read_yaml_config, prepare_export_path
from .global_configs import cleanup_global_configs, process_global_configs, \
    export_global_configs
from .upstreams import process_upstreams, cleanup_upstreams, export_upstreams
from .page_rules import process_page_rules, cleanup_page_rules, export_page_rules
from .edgelang_rules import process_edgelang_rules, cleanup_edgelang_rules, export_edgelang_rules
from .global_lua_modules import cleanup_global_lua_modules, \
    process_global_lua_modules, export_global_lua_modules
from .basic_auth_groups import process_basic_auth_groups, export_basic_auth_groups
from .users import process_users, export_users_and_groups
from .edge_email import read_email_configs
from .k8s_upstreams import process_k8s_upstreams, cleanup_k8s_upstreams, export_k8s_upstreams
from .global_upstreams import process_global_upstreams, cleanup_global_upstreams, export_global_upstreams
from .global_k8s_upstreams import process_global_k8s_upstreams, \
    cleanup_global_k8s_upstreams, export_global_k8s_upstreams
from .user_variables import process_user_variables, cleanup_user_variables, export_user_variables
from .ip_lists import process_ip_lists, cleanup_ip_lists, export_ip_lists
from .global_ip_lists import process_global_ip_lists, cleanup_global_ip_lists, export_global_ip_lists
from .waf_whitelist import process_waf_whitelist, cleanup_waf_whitelist, export_waf_whitelist
from .global_custom_actions import process_global_custom_actions, \
    cleanup_global_custom_actions, export_global_custom_actions
from .global_page_templates import process_global_page_templates, \
    cleanup_global_page_templates, export_global_page_templates
from .global_basic_auth_groups import process_global_basic_auth_groups, \
    cleanup_global_basic_auth_groups, export_global_basic_auth_groups
from .global_static_files import process_global_static_files, \
    cleanup_global_static_files, export_global_static_files
from .k8s import process_k8s_clusters, cleanup_k8s_clusters, export_k8s_clusters

# sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

supported_locations = {
    'page_rules': {
        'process_functions': [ process_http_app, process_page_rules ],
        'cleanup_functions': [ cleanup_page_rules ],
        'export_functions': [ export_page_rules ],
    },
    'edgelang_rules': {
        'process_functions': [ process_http_app, process_edgelang_rules ],
        'cleanup_functions': [ cleanup_edgelang_rules ],
        'export_functions': [ export_edgelang_rules ],
    },
    'upstreams': {
        'process_functions': [ process_http_app, process_upstreams ],
        'cleanup_functions': [ cleanup_upstreams ],
        'export_functions': [ export_upstreams ],
    },
    'k8s_upstreams': {
        'process_functions': [ process_http_app, process_k8s_upstreams ],
        'cleanup_functions': [ cleanup_k8s_upstreams ],
        'export_functions': [ export_k8s_upstreams ],
    },
    'user_variables': {
        'process_functions': [ process_http_app, process_user_variables ],
        'cleanup_functions': [ cleanup_user_variables ],
        'export_functions': [ export_user_variables ],
    },
    'ip_lists': {
        'process_functions': [ process_http_app, process_ip_lists ],
        'cleanup_functions': [ cleanup_ip_lists ],
        'export_functions': [ export_ip_lists ],
    },
    'waf_whitelist': {
        'process_functions': [ process_http_app, process_waf_whitelist ],
        'cleanup_functions': [ cleanup_waf_whitelist ],
        'export_functions': [ export_waf_whitelist ],
    },
    'basic_auth_groups': {
        'process_functions': [ process_http_app, process_basic_auth_groups ],
        'cleanup_functions': [],    # TODO
        'export_functions': [ export_basic_auth_groups ],
    },
    'global_lua_modules': {
        'process_functions': [ process_global_lua_modules ],
        'cleanup_functions': [ cleanup_global_lua_modules ],
        'export_functions': [ export_global_lua_modules ],
        'check_domain': False,
    },
    'global_custom_actions': {
        'process_functions': [ process_global_custom_actions ],
        'cleanup_functions': [ cleanup_global_custom_actions ],
        'export_functions': [ export_global_custom_actions ],
        'check_domain': False,
    },
    'global_configs': {
        'process_functions': [ process_global_configs ],
        'cleanup_functions': [ cleanup_global_configs ],
        'export_functions': [ export_global_configs ],
        'check_domain': False,
    },
    'global_page_templates': {
        'process_functions': [ process_global_page_templates ],
        'cleanup_functions': [ cleanup_global_page_templates ],
        'export_functions': [ export_global_page_templates ],
        'check_domain': False,
    },
    'global_upstreams': {
        'process_functions': [ process_global_upstreams ],
        'cleanup_functions': [ cleanup_global_upstreams ],
        'export_functions': [ export_global_upstreams ],
        'check_domain': False,
    },
    'global_k8s_upstreams': {
        'process_functions': [ process_global_k8s_upstreams ],
        'cleanup_functions': [ cleanup_global_k8s_upstreams ],
        'export_functions': [ export_global_k8s_upstreams ],
        'check_domain': False,
    },
    'global_ip_lists': {
        'process_functions': [ process_global_ip_lists ],
        'cleanup_functions': [ cleanup_global_ip_lists ],
        'export_functions': [ export_global_ip_lists ],
        'check_domain': False,
    },
    'global_basic_auth_groups': {
        'process_functions': [ process_global_basic_auth_groups ],
        'cleanup_functions': [ cleanup_global_basic_auth_groups ],
        'export_functions': [ export_global_basic_auth_groups ],
        'check_domain': False,
    },
    'global_static_files': {
        'process_functions': [ process_global_static_files ],
        'cleanup_functions': [ cleanup_global_static_files ],
        'export_functions': [ export_global_static_files ],
        'check_domain': False,
    },
    'k8s': {
        'process_functions': [ process_k8s_clusters ],
        'cleanup_functions': [ cleanup_k8s_clusters ],
        'export_functions': [ export_k8s_clusters ],
        'check_domain': False,
    },
    'users': {
        'process_functions': [ process_users ],
        'cleanup_functions': [],    # do not support
        'export_functions': [ export_users_and_groups ],
        'check_domain': False,
    },
    'all': {
        'process_functions': [
            process_k8s_clusters,
            process_global_configs,
            process_global_lua_modules,
            process_global_basic_auth_groups,
            process_global_upstreams,
            process_global_k8s_upstreams,
            process_global_ip_lists,
            process_global_page_templates,
            process_global_static_files,
            process_global_custom_actions,
            process_http_app,
            # TODO add/update before updating page rules, delete after page rules
            process_upstreams,
            process_k8s_upstreams,
            process_basic_auth_groups,
            process_user_variables,
            process_ip_lists,
            process_waf_whitelist,
            process_edgelang_rules,
            process_page_rules,
            process_users,
        ],
        'cleanup_functions': [
            cleanup_page_rules,     # for compatibility with older versions(<24.9.1) of Edge.
            cleanup_http_app,
            cleanup_global_configs,
            cleanup_global_lua_modules,
            cleanup_global_custom_actions,
            cleanup_global_basic_auth_groups,
            cleanup_global_upstreams,
            cleanup_global_k8s_upstreams,
            cleanup_global_ip_lists,
            cleanup_global_page_templates,
            cleanup_global_static_files,
            cleanup_k8s_clusters,
        ],
        'export_functions': [
            export_app_config,
            export_page_rules,
            export_edgelang_rules,
            export_upstreams,
            export_k8s_upstreams,
            export_user_variables,
            export_ip_lists,
            export_waf_whitelist,
            # NOTE: do not export basic auth users by default
            export_basic_auth_groups,
            export_global_basic_auth_groups,
            export_global_custom_actions,
            export_global_lua_modules,
            export_global_upstreams,
            export_global_k8s_upstreams,
            export_global_ip_lists,
            export_global_page_templates,
            export_global_configs,
            export_global_static_files,
            export_k8s_clusters,
            export_users_and_groups,
        ],
    },
}

def parse_args():
    description = "Update or add OpenResty Edge configuration."
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument("-t", "--api-token", dest="api_token", action="store", required=True,
                        help="specify the API token for sending the request")
    parser.add_argument("-u", "--admin-url", dest="admin_url", action="store", required=True,
                        help="specify the URL of the OpenResty Edge Admin. For example, https://admin.com:443")
    parser.add_argument("-s", "--disable-ssl-verify", dest="disable_ssl_verify", action="store_true", default=False,
                        help="turn off SSL verification for requests to access OpenResty Edge Admin")
    parser.add_argument("-i", "--configs-path", dest="configs_path", action="store", required=False,
                        help="specify the path to the configuration file")
    parser.add_argument("-e", "--email-config-file", dest="email_config_file", action="store", required=False,
                        help="specify the file to the email configuration; if not specified, the email.yaml file in the configuration path will be used")
    parser.add_argument("-U", "--users-config-path", dest="users_config_path", action="store", required=False,
                        help="specify the path to the users configuration; if not specified, the users/ path in the configuration path will be used")
    parser.add_argument("-F", "--export-fake-privacy-info", dest="export_fake_info", action="store_true", required=False,
                        help="use placeholders in place of privacy information when exporting")
    parser.add_argument("-S", "--staging-release", dest="staging_release", action="store_true", default=False,
                        help="only release the configuration to the staging gateway server")

    keys_string = ', '.join(supported_locations.keys())
    parser.add_argument("-l", "--location", dest="location", action="store",
                        help=f"specify the configuration name that needs to be updated, supported: {keys_string}")
    parser.add_argument("-d", "--domain", dest="domain", action="store",
                        help="specify a domain name. When an HTTP application containing this domain exists, it will be updated; otherwise, a new application will be created")
    parser.add_argument("-P", "--export-to-path", dest="export_to_path", action="store",
                        help="specify the storage path of the exported configuration. If not specified, a directory prefixed with configs will be automatically created in the current path.")


    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument("-p", "--partition-id", dest="partition_id", action="store", type=int,
                        help="specify the id of the partition where you want to add or update the configuration")
    group.add_argument("-n", "--partition-name", dest="partition_name", action="store",
                        help="specify the name of the partition where you want to add or update the configuration")

    action_group = parser.add_mutually_exclusive_group(required=False)
    action_group.add_argument("-c", "--cleanup", dest="cleanup", action="store_true", default=False,
                        help="based on the location. This option allows for the cleanup of page rules, application upstreams, global user variables, and resetting the access log format for partitions. It can also be used independently of the location")
    action_group.add_argument("-E", "--export", dest="export", action="store_true", default=False,
                        help="export the configuration to the specified path, which can be specified through the --export-to-path option")

    args = parser.parse_args()

    location = args.location
    if location is not None and location not in supported_locations:
        parser.error(f"unsupported location: {location}.\n\nAvaiable locations:\n{keys_string}")

    ssl_verify = True
    if args.disable_ssl_verify is True:
        ssl_verify = False
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        requests.packages.urllib3.disable_warnings()

    client = Edge2Client(args.admin_url, None, None, args.api_token)
    client.set_ssl_verify(ssl_verify)

    data = client.get_sync_to_all()
    if not client.request_ok():
        error("Failed to get sync to all flag from OpenResty Edge Admin")

    ngx_sync_to_all = data.get('ngx', True) if data else True
    lua_module_sync_to_all = data.get('lua_module', True) if data else True

    check_users_only = False
    if (location is not None and location == "users") \
    and (args.users_config_path is not None):
        check_users_only = True
    else:
        if not args.export and not args.cleanup:
            configs_path = args.configs_path
            if configs_path is None:
                parser.error("the following arguments are required: -i/--configs-path")

            if not os.path.exists(configs_path):
                error(f"configs path not exists: {configs_path}")

            if not os.path.isdir(configs_path):
                error(f"configs path is not a directory: {configs_path}")

        if not ngx_sync_to_all or not lua_module_sync_to_all:
            if args.partition_id is None and args.partition_name is None:
                parser.error("the following arguments are required: -p/--partition-id or -n/--partition-name")

        if not (location is not None and supported_locations[location].get('check_domain', True) is False) and args.domain is None:
            parser.error("the following arguments are required: -d/--domain")

    email_config_file = args.email_config_file
    if email_config_file and not os.path.exists(email_config_file):
        error(f"sender's email config file not exists: {email_config_file}")

    users_config_path = args.users_config_path
    if users_config_path and not os.path.exists(users_config_path):
        error(f"users config path not exists: {users_config_path}")

    staging_release = False
    if args.staging_release:
        staging_release = True

    return {
        'args': args,
        'check_users_only': check_users_only,
        'client': client,
        'ngx_sync_to_all': ngx_sync_to_all,
        'lua_module_sync_to_all': lua_module_sync_to_all,
        'export_fake_info': args.export_fake_info,
        'staging_release': staging_release
    }

def cleanup(ctx):
    location = ctx['location']
    if not location:
        location = "all"

    export_functions = supported_locations[location]['cleanup_functions']

    for func in export_functions:
        try:
            func(ctx)
        except Exception as e:
            error(f"failed to cleanup", e)

def export(ctx):
    location = ctx['location']
    if not location:
        location = "all"
        ctx['export_users'] = False
    else:
        ctx['export_users'] = True

    export_functions = supported_locations[location]['export_functions']

    for func in export_functions:
        try:
            func(ctx)
        except Exception as e:
            error(f"failed to export", e)

def import_to_edge(ctx):
    location = ctx['location']
    if not location:
        location = "all"

    export_functions = supported_locations[location]['process_functions']

    for func in export_functions:
        try:
            func(ctx)
        except Exception as e:
            error(f"failed to import", e)

def check_partition(client, partition_id, partition_name):
    if partition_id:
        data = client.get_cluster_group(partition_id)
        if not data:
            error(f'partition not found, partition id: {partition_id}')

        return partition_id

    if partition_name:
        partitions = client.get_all_cluster_groups()
        for p in partitions:
            if p['name'] == partition_name:
                # get partition_id
                return p['id']

        error(f'partition not found, partition name: {partition_name}')

def main(args=None):
    ctx = parse_args()

    args = ctx['args']
    check_users_only = ctx['check_users_only']
    client = ctx['client']
    ngx_sync_to_all = ctx['ngx_sync_to_all']

    # versions = client.get_version()
    # if versions is None or versions.get('product_type', None) is None:
    #     error("get openresty edge product type failed")

    # if versions['product_type'] != 'enterprise':
    #     error("openresty edge product type is not enterprise")

    location = args.location
    configs_path = args.configs_path

    ctx['location'] = location
    ctx['configs_path'] = configs_path

    email_config_file = args.email_config_file
    if email_config_file is None and configs_path is not None:
        email_config_file = f"{configs_path}/email.yaml"

    users_config_path = args.users_config_path
    if users_config_path is None and configs_path is not None:
        users_config_path = f"{configs_path}/users"

    ctx['users_config_path'] = users_config_path

    email_configs = read_email_configs(email_config_file)

    ctx['email_configs'] = email_configs

    if ngx_sync_to_all is False:
        partition_id = args.partition_id
        partition_name = args.partition_name
        if check_users_only != True:
            partition_id = check_partition(client, partition_id, partition_name)
    else:
        # default partition is 1
        partition_id = 1
        partition_name = "default"

    ctx['partition_id'] = partition_id
    ctx['partition_name'] = partition_name

    domain = args.domain
    ctx['domain'] = domain

    if args.export:
        export_configs_path = args.export_to_path
        if export_configs_path is None:
            export_configs_path = prepare_export_path(partition_id, domain)
            info(f"export configs path: {export_configs_path}")

        ctx['export_to_path'] = export_configs_path

        export(ctx)
        print("[!] Export Finished.")
        sys.exit()

    if args.cleanup:
        cleanup(ctx)
        print("[!] Cleanup Finished.")
        sys.exit()

    import_to_edge(ctx)
    print("[!] Import Finished.")

if __name__ == "__main__":
    exit(main())



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
update rule condition values.
"""

import json
import argparse
from utils_common import proc_cond, proc_conseq, \
        proc_waf, proc_proxy, proc_cache, proc_content

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def update_cond(cond, vals, type):
    values = []

    for val in vals:
        values.append({'val': val, 'type': type})

    new_cond = cond.copy()
    new_cond['values'] = values
    return new_cond


def main(args):
    client = py_client.get_client()
    client.use_app(args.app_id)

    rule = client.get_rule(args.rule_id)
    rule_conds = rule.get('conditions')

    if rule_conds is None:
        print('>>> ERR: there\'s no conditions to modify in the rule \'{}\''
              .format(args.rule_id))
        exit(1)

    conds = []
    if len(rule_conds) == 1:
        rule_cond = rule_conds[0]
        conds.append(update_cond(rule_cond, args.ip, args.val_type))

    elif args.var_name:
        for rule_cond in rule_conds:
            if args.var_name == rule_cond.get('variable', {}).get('name'):
                conds.append(update_cond(rule_cond, args.ip, args.val_type))
            else:
                conds.append(rule_cond)

    if args.dry_run:
        print('old rule condtions:')
        print(json.dumps(rule_conds))
        print('new rule condtions:')
        print(json.dumps(conds))
        print('exiting...')
        exit(0)

    print('update rule conditions...')
    # update rule_conds.
    client.put_rule(
            rule_id=args.rule_id,
            condition=proc_cond(conds),
            conseq=proc_conseq(rule.get('actions')),
            waf=proc_waf(rule.get('waf')),
            cache=proc_cache(rule.get('cache')),
            proxy=proc_proxy(rule.get('proxy')),
            content=proc_content(rule.get('content')),
            top=rule.get('top'),
            last=rule.get('last'),
            order=rule.get('order')
        )
    print('done.')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='update rule\'s condition value.')

    parser.add_argument('--dry-run',
                        dest='dry_run',
                        action='store_true',
                        help='dry run.')

    parser.add_argument('--app-id', type=int, dest='app_id',
                        help='Application id', required=True)
    parser.add_argument('--rule-id', type=int, dest='rule_id',
                        help='Rule id', required=True)

    parser.add_argument('--var-name', type=str, dest='var_name',
                        help='Specify a vairable name to '
                        'select among multiple condtions: '
                        '[client-addr first-x-forwarded-addr '
                        'last-x-forwarded-addr]')
    parser.add_argument('--val-type', type=str, dest='val_type',
                        help='Value type: [str rx wc netaddr]',
                        default='netaddr')
    parser.add_argument('--print-rules', dest='print_rules',
                        help=argparse.SUPPRESS, action='store_true')

    parser.add_argument('ip', metavar='IP', type=str, nargs='+',
                        help='set condition ip.')

    args = parser.parse_args()

    main(args)



import os

from .utils import error, warn, info, get_md5_from_comment, \
    cal_config_md5, line, get_real_comment, get_app_id_by_domain
from .read_config import read_yaml_config, write_yaml_config, add_before_comment

def check_basic_auth_groups(groups, filename):
    if not isinstance(groups, dict) and not isinstance(groups, list):
        error("unsupported file format for basic auth groups, file: {filename}")

    if isinstance(groups, dict):
        groups = [ groups ]

    for group in groups:
        if 'name' not in group:
            error("basic auth group name not found, file: {filename}")

        if 'users' not in group:
            error("basic auth group users not found, file: {filename}")

        label = group.get('label', '')
        group_name = group['name']
        if not isinstance(label, str):
            error(f"basic auth group label must be a string, file: {filename}, line: {line(label)}")

        users = group['users']
        for user in users:
            username = user.get("username", None)
            password = user.get("password", None)

            if not isinstance(username, str) or not username:
                error(f"invalid username in basic auth group {group_name}: {username}, file: {filename}, line: {line(user)}")

            if not isinstance(password, str) or not password:
                error(f"invalid password in basic auth group {group_name}: {password}, file: {filename}, line: {line(user)}")

    return True

def update_basic_auth_users(client, app_id, group_id, users):
    # get all users
    old_users = client.get_app_basic_auth_users_in_group(group_id, app_id=app_id)

    old_users_map = dict()
    for u in old_users:
        old_users_map[u['username']] = u

    for user in users:
        username = user['username']
        password = user['password']

        if username in old_users_map:
            # update
            try:
                info(f"updating basic auth user \"{username}\" to group, app id: {app_id}, group id: {group_id}")
                user_id = old_users_map[username]['id']
                client.put_app_basic_auth_user(user_id, group_id, username, password, app_id=app_id)
            except Exception as e:
                error(f"failed to update basic auth user, file: {filename}, line: {line(user)}", e)

            del(old_users_map[username])
        else:
            # add
            try:
                info(f"adding basic auth user \"{username}\" to group, app id: {app_id}, group id: {group_id}")
                client.new_app_basic_auth_user(group_id, username, password, app_id=app_id)
            except Exception as e:
                error(f"failed to add basic auth user, file: {filename}, line: {line(user)}", e)
            pass

    # delete
    for username, user in old_users_map.items():
        try:
            info(f"deleting basic auth user \"{username}\" from group, app id: {app_id}, group id: {group_id}")
            client.del_app_basic_auth_user(user["id"], group_id, app_id=app_id)
        except Exception as e:
            error(f"failed to delete basic auth user, file: {filename}, line: {line(user)}", e)


def basic_auth_group_changed(old_group, new_md5):
    if 'label' not in old_group:
        return True

    old_md5 = get_md5_from_comment(old_group['label'])
    if new_md5 and old_md5 == new_md5:
        return False

    return True

def process_basic_auth_groups(ctx):
    client = ctx['client']
    app_id = ctx['app_id']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "basic_auth_groups"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    # pre check
    info("Checking if basic auth groups are valid")
    for filename, groups in configs.items():
        check_basic_auth_groups(groups, filename)

    info("Checking if basic auth groups have changed")
    old_groups = client.get_all_app_basic_auth_user_groups(app_id)
    old_groups_map = dict()

    for g in old_groups:
        old_groups_map[g['name']] = g

    for filename, new_groups in configs.items():
        if isinstance(new_groups, dict):
            new_groups = [ new_groups ]
        for new_group in new_groups:
            # check if group exists
            name = new_group['name']
            md5 = cal_config_md5(new_group)

            if name not in old_groups_map:
                try:
                    info(f"adding basic auth group \"{name}\" to app, app id: {app_id}")
                    label = new_group.get('label', '')
                    label = f"{label}md5: {md5}, please do not modify."
                    group_id = client.new_app_basic_auth_user_group(name, label=label, app_id=app_id)
                    update_basic_auth_users(client, app_id, group_id, new_group['users'])
                except Exception as e:
                    error(f"failed to add basic auth group to app, file: {filename}, line: {line(new_group)}", e)

                # process next
                continue

            # group exists, check if group changed
            if basic_auth_group_changed(old_groups_map[name], md5):
                label = new_group.get('label', '')
                if label:
                    label = f"{label}md5: {md5}, please do not modify."
                else:
                    label = old_groups_map[name].get('label', '')
                    if label:
                        label = get_real_comment(label)
                        label = f"{label}md5: {md5}, please do not modify."
                    else:
                        label = f"md5: {md5}, please do not modify."

                try:
                    group_id = old_groups_map[name]['id']
                    update_basic_auth_users(client, app_id, group_id, new_group['users'])
                    info(f"updating basic auth group \"{name}\" to app, app id: {app_id}")
                    client.put_app_basic_auth_user_group(group_id, name, label=label, app_id=app_id)
                except Exception as e:
                    error(f"failed to update basic auth group to app, file: {filename}, line: {line(new_group)}", e)

            del(old_groups_map[name])

    # delete
    for name, group in old_groups_map.items():
        try:
            info(f"deleting basic auth group \"{name}\" from app, app id: {app_id}")
            client.del_app_basic_auth_user_group(group["id"], app_id=app_id)
        except Exception as e:
            error(f"failed to delete basic auth group to app, group id: {group['id']}, app id {app_id}", e)

def export_basic_auth_groups(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    domain = ctx['domain']
    configs_path = ctx['export_to_path']
    export_users = ctx.get('export_users', False)

    """
    Since passwords cannot be exported, not even users are exported by default

    Args:
        export_users (bool, optional): Whether to export users info. Defaults to False.
    """
    if ctx.get('app_not_found', False) is True:
        return

    app_id = ctx.get('app_id', None)
    if app_id is None:
        app_id = get_app_id_by_domain(client, partition_id, domain)
        if not app_id:
            warn(f"App not found, partition_id: {partition_id}, domain: {domain}")
            return

        ctx['app_id'] = app_id

    groups = client.get_all_app_basic_auth_user_groups(app_id)
    if not groups:
        info(f"No basic auth groups found for app_id: {app_id}")
        return

    formatted_groups = []
    for group in groups:
        formatted_group = {
            'name': group['name'],
            'label': group.get('label', ''),
            'users': []
        }

        # Get users for this group
        if export_users:
            password = ''
            if ctx['export_fake_info']:
                password = "********"

            users = client.get_app_basic_auth_users_in_group(group['id'], app_id=app_id)
            for user in users:
                formatted_user = {
                    'username': user['username'],
                    'password': password
                }
                formatted_user = add_before_comment(formatted_user, 'password',
                                                       "password: We don't export actual password for security reasons")
                formatted_group['users'].append(formatted_user)

        formatted_groups.append(formatted_group)

    export_path = os.path.join(configs_path, 'basic_auth_groups')

    try:
        for g in formatted_groups:
            group_name = g['name']
            write_yaml_config(export_path, f"{group_name}.yaml", [g])
            info(f"Basic auth group {group_name} exported successfully to basic_auth_groups/{group_name}.yaml")
        info(f"All basic auth groups exported successfully to basic_auth_groups/")
    except Exception as e:
        error(f"Failed to export basic auth groups to basic_auth_groups/", e)



import os
import io

from ruamel.yaml import YAML
from ruamel.yaml.scalarstring import LiteralScalarString

from .utils import error, warn, info, line

def read_yaml_file(file):
    yaml = YAML()
    with open(file, 'r') as f:
        config = yaml.load(f)

    return config

def read_yaml_config(configs_path, keyword=None):
    yaml = YAML()
    yaml_config = {}

    file_path = None
    directory_path = configs_path
    if keyword is not None:
        file_path = f"{configs_path}/{keyword}.yaml"
        directory_path = f"{configs_path}/{keyword}"

        if os.path.isfile(file_path) and os.path.isdir(directory_path):
            warn(f"the file and directory both exist; prioritize using the file: {file_path}")

    # check if file exists
    if file_path is not None and os.path.isfile(file_path):
        with open(file_path, 'r') as f:
            config = yaml.load(f)

        yaml_config[f"{keyword}.yaml"] = config
        return yaml_config

    res = dict()
    # check if directory exists
    if os.path.isdir(directory_path):
        for filename in os.listdir(directory_path):
            if filename.endswith('.yaml'):
                full_path = os.path.join(directory_path, filename)
                with open(full_path, 'r') as f:
                    yaml_config[filename] = yaml.load(f)
        return yaml_config

    # not found
    return None

def read_lua_modules(configs_path, folder):
    folder = f"{configs_path}/{folder}"
    if not os.path.isdir(folder):
        # not found
        return None

    lua_files_content = {}
    for filename in os.listdir(folder):
        if filename.endswith('.lua'):
            file_key = os.path.splitext(filename)[0]
            file_path = os.path.join(folder, filename)
            with open(file_path, 'r', encoding='utf-8') as file:
                lua_files_content[file_key] = file.read()

    return lua_files_content


def write_yaml_config(configs_path, filename, config=None):
    yaml = YAML()
    yaml.default_flow_style = False
    yaml.indent(mapping=2, sequence=2, offset=0)

    if not os.path.exists(configs_path):
        os.makedirs(configs_path)

    file_path = os.path.join(configs_path, filename)

    with open(file_path, 'w') as f:
        yaml.dump(config, f)

    return file_path


from datetime import datetime

def prepare_export_path(partition_id, domain):
    current_path = os.getcwd()

    current_datetime = datetime.now().strftime("%Y%m%d_%H%M%S")

    dir_name = f"oredge-configs-{partition_id}-{domain}-{current_datetime}"

    full_path = os.path.join(current_path, dir_name)

    os.makedirs(full_path, exist_ok=True)

    return full_path

def format_yaml_content(content):
    return LiteralScalarString(content)

def add_before_comment(data, key, comment, indent=2):
    yaml = YAML()

    stream = io.StringIO()
    yaml.dump(data, stream)
    yaml_str = stream.getvalue()

    loaded_data = yaml.load(yaml_str)

    loaded_data.yaml_set_comment_before_after_key(key, before=comment, indent=indent)

    return loaded_data



#!/usr/bin/python3
# -*- coding: UTF-8 -*-
"""
update dns record
"""

import argparse
import sys
# import json


try:
    sys.path.append('/usr/local/oredge-python-sdk/lib')
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def parse_records(record):
    record_list = record.split(':', 1)
    if not record_list:
        raise argparse.ArgumentTypeError("failed to parse record")

    return record_list


def yes_or_no(question):
    reply = str(input(question+' (y/n): ')).lower().strip()
    if reply[0] == 'y':
        return True
    if reply[0] == 'n':
        return False
    else:
        return yes_or_no("Uhhhh... please enter ")


def main(args):
    client = py_client.get_client()

    dns_app = client.get_dns_app(args.dns_id)
    if not dns_app:
        raise Exception('DNS app not found:', args.dns_id)

    if not yes_or_no('dns zone: ' + dns_app.get('zone')):
        print("exiting ...")
        exit(0)

    client.use_dns_app(args.dns_id)

    args_records = {}
    for args_record in args.records:
        args_records[args_record[0]] = args_record[1]

    dns_records = client.get_dns_records()

    new_record_list = []
    update_record_list = []
    update_record_exists = {}

    # records to be update
    for dns_record in dns_records.get('data'):
        if dns_record.get('type') == 'A':
            sub_domain = dns_record.get('sub_domain')
            args_record_ip = args_records.get(sub_domain)

            if args_record_ip == dns_record.get('ip'):
                update_record_exists[sub_domain] = True

                print("info: skip update this record: ",
                      sub_domain, ":", args_record_ip)
                continue

            if args_record_ip:
                update_record_list.append({
                    'record_id': dns_record.get('id'),
                    'record_type': 'A',
                    'sub_domain': dns_record.get('sub_domain'),
                    'ip': args_record_ip
                })

                update_record_exists[sub_domain] = True

    for sub_domain, ip in args_records.items():
        if not update_record_exists.get(sub_domain):
            new_record_list.append({
                'record_type': 'A',
                'sub_domain': sub_domain,
                'ip': ip
            })

    for record in new_record_list:
        print("append new record: %s %s %s" % (record.get('sub_domain'),
              record.get('record_type'), record.get('ip')))

        if args.dry_run:
            print('dry run ...')
        else:
            client.new_dns_record(sub_domain=record.get('sub_domain'),
                                  record_type=record.get('record_type'),
                                  ip=record.get('ip'))

    for record in update_record_list:
        print("update record: %s %s %s" % (record.get('sub_domain'),
              record.get('record_type'), record.get('ip')))

        if args.dry_run:
            print('dry run ...')
        else:
            client.put_dns_record(sub_domain=record.get('sub_domain'),
                                  record_type=record.get('record_type'),
                                  ip=record.get('ip'),
                                  record_id=record.get('record_id'))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='update dns A record')

    parser.add_argument('--dry-run',
                        dest='dry_run',
                        action='store_true',
                        help='dry run.')
    parser.add_argument('--dns-id',
                        dest='dns_id',
                        type=int,
                        required=True)
    parser.add_argument('records', metavar='SUB_DOMAIN:IP', nargs='+',
                        type=parse_records, help='dns records')
    args = parser.parse_args()

    main(args)



import re

from .utils import error, warn, info, to_fake_name, line, to_real_name, extract_name

def replace_global_variables(client, content, partition_id):
    if not isinstance(content, str):
        raise Exception("bad content")

    variables = []
    pattern = r'\$or_global_user_variable_[-a-zA-Z0-9_]+'
    variables = re.findall(pattern, content)

    variable_names = {}
    for var in variables:
        match = re.search(r'\$or_global_user_variable_(.*)', var)
        # should always match
        name = match.group(1)
        variable_names[var] = name

    global_variables = client.get_all_global_vars()
    if not isinstance(global_variables, list):
        global_variables = list()

    global_variable_names = {}
    for var in global_variables:
        global_variable_names[var['name']] = var['id']

    data = client.get_sync_to_all()
    ngx_sync_to_all = True
    if data and data.get('ngx', True) == False:
        ngx_sync_to_all = False

    for fake_name, name in variable_names.items():
        if name in global_variable_names:
            var_id = global_variable_names[name]
        else:
            # for compatibility with older versions of this tool
            real_name = to_real_name(name, partition_id, ngx_sync_to_all)
            if real_name in global_variable_names:
                var_id = global_variable_names[real_name]
            else:
                raise Exception(f"global variable not found in Edge Admin: {name}")

        content = content.replace(fake_name, f'$or-global-{var_id}')

    return content

def replace_global_variable_in_el(client, actions, filename, partition_id):
    if not isinstance(actions, list) and not isinstance(actions, dict):
        return

    if not isinstance(actions, list) and isinstance(actions, dict):
        new_actions = list()
        for k, v in actions.items():
            new_actions.append({k: v})
        actions = new_actions

    for action in actions:
        if not isinstance(action, dict) or 'user-code' not in action:
            continue

        el_value = ''
        user_code_data = action['user-code']
        if isinstance(user_code_data, dict) and 'el' in user_code_data:
            el_value = user_code_data['el']
        else:
            if user_code_data:
                error(f"bad action in page rule, file: {filename}, line: {line(user_code_data)}")
            else:
                error(f"bad action in page rule, file: {filename}, line: {line(user_code_data)}")

        try:
            user_code_data['el'] = replace_global_variables(client, el_value, partition_id)
        except Exception as e:
            error(f"failed to replace global variable in Edgelang, file: {filename}, line: {line(user_code_data)}", e)

def replace_global_variable_in_log_formats(client, log_formats, filename, partition_id):
    for name, fmt in log_formats.items():
        try:
            log_format = fmt['format']
            fmt['format'] = replace_global_variables(client, log_format, partition_id)
        except Exception as e:
            error(f"failed to replace global variable in access log format, file: {filename}, log format: {name}", e)

def restore_global_variable_names(client, content, partition_id):
    if not isinstance(content, str):
        raise Exception("Bad content")

    # Find all global variable IDs in the content
    pattern = r'\$or-global-[0-9]+'
    variables = re.findall(pattern, content)

    # Get all global variables
    global_variables = client.get_all_global_vars()
    if not isinstance(global_variables, list):
        global_variables = []

    # Create a mapping of variable IDs to names
    global_variable_ids = {var['id']: var['name'] for var in global_variables}

    # Replace each variable ID with its corresponding name
    for var in variables:
        match = re.search(r'\$or-global-(\d+)', var)
        if match:
            var_id = int(match.group(1))
            if var_id in global_variable_ids:
                var_name = global_variable_ids[var_id]
                # NAME_for_partition_ID to NAME
                var_name = extract_name(var_name)
                content = content.replace(var, f'$or_global_user_variable_{var_name}')
            else:
                warn(f"Global variable ID not found: {var_id}")

    return content

def restore_global_variable_names_in_el(client, actions, partition_id):
    if not isinstance(actions, list) and not isinstance(actions, dict):
        return

    if not isinstance(actions, list) and isinstance(actions, dict):
        new_actions = list()
        for k, v in actions.items():
            new_actions.append({k: v})
        actions = new_actions

    for action in actions:
        if not isinstance(action, dict) or 'user-code' not in action:
            continue

        user_code_data = action['user-code']
        if isinstance(user_code_data, dict) and 'el' in user_code_data:
            el_value = user_code_data['el']
            try:
                user_code_data['el'] = restore_global_variable_names(client, el_value, partition_id)
            except Exception as e:
                error(f"Failed to restore global variable names in Edgelang", e)

def restore_global_variable_names_in_log_formats(client, log_formats, partition_id):
    new_log_formats = list()
    for fmt in log_formats:
        name = fmt['name']
        try:
            log_format = fmt['format']
            new_fmt = {
                'name': fmt['name'],
                'format': restore_global_variable_names(client, log_format, partition_id),
                'default': fmt['default']
            }

            new_log_formats.append(new_fmt)
        except Exception as e:
            error(f"Failed to restore global variable names in access log format, log format: {name}", e)

    return new_log_formats



import json
import os

from .utils import error, warn, info, line, get_updated_list, to_fake_name, \
    clear_partition_changes, release_partition_changes, to_real_name
from .read_config import read_yaml_config, write_yaml_config
from .global_variables import replace_global_variable_in_log_formats, \
    restore_global_variable_names_in_log_formats

def reset_access_log_format(client, partition_id):
    partition_ngx_conf = client.get_partition_ngx_config(partition_id, detail=True)
    if 'access_log_formats' not in partition_ngx_conf:
        error(f"access log formats not found in Edge Admin, partition id: {partition_id}")

    main_id = None
    for fmt in partition_ngx_conf['access_log_formats']:
        if fmt.get('name', False) == "main":
            main_id = fmt['id']

    config = dict()
    config['access_log_formats'] = [
        {
            'id': main_id,
            'name': "main",
            'default': True,
            'format': "$remote_addr - $remote_user [$time_local] $http_host \"$request\" $status $body_bytes_sent $request_time \"$http_referer\" \"$http_user_agent\" $upstream_addr $upstream_status $upstream_response_time",
        }];

    try:
        info(f"reset access log format, partition id: {partition_id}")
        client.set_partition_ngx_config(config, partition_id)
    except Exception as e:
        clear_partition_changes(client, partition_id)
        error(f"failed to reset access log formats, partition id: {partition_id}", e)

    # release partition changes
    release_partition_changes(client, partition_id)

def read_global_config(configs_path, location, config_key=None):
    global_configs = read_yaml_config(configs_path, location)
    if global_configs is not None:
        filename = f"{location}.yaml"
        configs = global_configs[filename]
        if config_key is not None:
            target_config = configs.get(config_key, None)
            return target_config
        else:
            return configs

    return None

def read_global_general_configs(configs_path, location):
    general_configs = read_global_config(configs_path, location, "general")
    return general_configs

def read_access_log_formats(configs_path, location, new_format_list):
    new_formats = dict()
    found_default = False

    if not new_format_list:
        new_format_list = read_global_config(configs_path, location, "access_log_formats")

    if not new_format_list:
        return new_formats, found_default

    for fmt in new_format_list:
        if 'name' not in fmt:
            error(f"missing name of access log format, file: {fmt.lc.filename}, line:{line(fmt)}")
        if 'format' not in fmt:
            error(f"missing format of access log format, file: {fmt.lc.filename}, line:{line(fmt)}")

        fmt['format'] = fmt['format'].replace("\n", "")
        if 'escape' in fmt and fmt['escape'] == "json":
            try:
                fmt['format'] = json.dumps(json.loads(fmt['format']))
            except:
                error(f"invalid access log format, unable to minimize json format, file: {fmt.lc.filename}, line:{line(fmt)}")

        # check if multiple default formats
        if fmt.get('default', False) == True:
            if found_default == True:
                error(f"multiple default access log formats, file: {fmt.lc.filename}, line:{line(fmt)}")
            else:
                found_default = True

        new_formats[fmt['name']] = fmt

    return new_formats, found_default

def read_global_custom_shared_zone(configs_path, location):
    zones = dict()
    zones_list = read_global_config(configs_path, location, "custom_shared_zone")
    if zones_list is None:
        return zones

    for v in zones_list:
        if 'name' not in v:
            error(f"missing name of access log format, file: {v.lc.filename}, line: {line(v)}")

        zones[v['name']] = v

    return zones

def read_global_user_variables(configs_path, location, partition_id):
    variables = dict()
    variables_list = read_global_config(configs_path, location, "user_variables")
    if variables_list is None:
        return variables

    for v in variables_list:
        if 'name' not in v:
            error(f"missing name of access log format, file: {v.lc.filename}, line: {line(v)}")

        var_name = v['name']
        variables[var_name] = v

    return variables

def process_global_configs(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_configs"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    # 1. process global variables
    info("Checking if global user variables have changed")
    new_variables = read_global_user_variables(configs_path, location, partition_id)
    if new_variables:
        old_variable_list = client.get_all_global_vars()
        updated_list = get_updated_list(new_variables, old_variable_list,
                                        "name", ["default", "type"], True)
        if updated_list:
            for var in updated_list:
                if 'id' in var:
                    try:
                        info(f"updating global user variable, variable id: {var['id']}, variable name: {var['name']}")
                        client.put_global_var(var_id=var['id'], name=var['name'],
                                        var_type=var['type'], default=var['default'])
                    except Exception as e:
                        error(f"failed to update global user variable", e)
                else:
                    try:
                        info(f"adding global user variable: {var['name']}")
                        client.new_global_var(name=var['name'], var_type=var['type'], default=var['default'])
                    except Exception as e:
                        error(f"failed to add global user variable", e)
    else:
        info("user variables not found in global_configs.yaml, skipping user variables...")

    # 2. process global general configs
    info("Checking if global general configs have changed")
    new_access_log_formats = None
    new_general_configs = read_global_general_configs(configs_path, location)
    if new_general_configs and 'access_log_formats' in new_general_configs:
        new_access_log_formats = new_general_configs['access_log_formats']
        del new_general_configs['access_log_formats']

    new_config = new_general_configs
    if not new_config:
        new_config = dict()

    # process access log formats
    new_formats, found_default = read_access_log_formats(configs_path, location, new_access_log_formats)
    if new_formats:
        # found log format in global_configs.yaml
        replace_global_variable_in_log_formats(client, new_formats,
                                            location + ".yaml", partition_id)
        partition_ngx_conf = client.get_partition_ngx_config(partition_id, detail=True)
        if 'access_log_formats' not in partition_ngx_conf:
            error(f"access log formats not found in Edge Admin, partition id: {partition_id}")

        if found_default:
            for fmt in partition_ngx_conf['access_log_formats']:
                if fmt.get('default', False) == True:
                    fmt['default'] = False

        # check if access log changed
        updated_list = get_updated_list(new_formats,
                                        partition_ngx_conf['access_log_formats'],
                                        "name",
                                        ["format", "escape", "default"])

        # update if change
        if updated_list:
            new_config['access_log_formats'] = updated_list
    else:
        info("access log formats not found in global_configs.yaml, skipping log format...")

    if new_config:
        try:
            info(f"updating global general configs, partition id: {partition_id}")
            client.set_partition_ngx_config(new_config, partition_id)
            # release partition changes
            release_partition_changes(client, partition_id)
        except Exception as e:
            clear_partition_changes(client, partition_id)
            error(f"failed to update global general configs, partition id: {partition_id}", e)

    # 3. process custom shared zone
    info("Checking if custom shared zones have changed")
    new_zones = read_global_custom_shared_zone(configs_path, location)
    if new_zones:
        global_ngx_conf = client.get_global_ngx_config(detail=True)

        old_shared_zone = list()
        if 'custom_shared_zone' in global_ngx_conf:
            old_shared_zone = global_ngx_conf['custom_shared_zone']

        updated_list = get_updated_list(new_zones,
                                        old_shared_zone,
                                        "name",
                                        ["size_unit", "size"])

        # update if change
        if updated_list:
            info("updating custom shared zone...")
            new_config = {
                'custom_shared_zone': updated_list
            }
            # global_ngx_conf['custom_shared_zone'] = updated_list
            client.set_global_ngx_config(new_config)
    else:
        info("custom shared zones not found in global_configs.yaml, skipping shared zones...")


def cleanup_global_configs(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    ngx_sync_to_all = ctx['ngx_sync_to_all']

    if ngx_sync_to_all:
        return

    # reset access log
    reset_access_log_format(client, partition_id)
    # do not delete global variables, supporting updates and inserts is enough
    release_partition_changes(client, partition_id)
    # do not delete custom shared zone
    # TODO check if custom shared zone using in global lua module (all partitions)

def export_global_configs(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['export_to_path']

    global_configs = {}

    # 1. Export user variables
    info("Exporting global user variables...")
    user_variables = client.get_all_global_vars()
    if user_variables:
        new_variables = list()
        for v in user_variables:
            name = v['name']

            # name == None means that the global variable does not belong to this partition
            if name is not None:
                new_v = {
                    'name': name,
                    'type': v.get('type', 'string'),
                    'default': v.get('default', '-'),
                }
                new_variables.append(new_v)

        global_configs['user_variables'] = new_variables

    # 2. Export access log formats
    info("Exporting global general configs...")
    partition_ngx_conf = client.get_partition_ngx_config(partition_id, detail=True)
    if 'id' in partition_ngx_conf:
        del partition_ngx_conf['id']

    partition_ngx_conf = dict(sorted(partition_ngx_conf.items()))

    global_configs['general'] = partition_ngx_conf
    if 'access_log_formats' in partition_ngx_conf:
        # replace global variable id to variable name
        formats = partition_ngx_conf['access_log_formats']
        access_log_formats = restore_global_variable_names_in_log_formats(client, formats, partition_id)
        global_configs['general']['access_log_formats'] = access_log_formats

    # 3. Export custom shared zones
    info("Exporting global custom shared zones...")
    global_ngx_conf = client.get_global_ngx_config(detail=True)
    if 'custom_shared_zone' in global_ngx_conf:
        global_configs['custom_shared_zone'] = global_ngx_conf['custom_shared_zone']

    export_path = configs_path

    # Write to YAML file
    try:
        write_yaml_config(export_path, "global_configs.yaml", global_configs)
        info(f"Global configs exported successfully to global_configs.yaml")
    except Exception as e:
        error(f"Failed to export global configs to global_configs.yaml", e)
